{ This unit defines the structure of the model. There are four functions. The
  first function, called counts, defines the number, names, and units of the
  model, the state variables, the process variables, the driver variables and
  the parameters. The second function, called processes, is the actual equations
  which make up the model. The third function, derivs, calculates the
  derivatives of state variables. And the fourth function, parcount, is used to
  automatically number the parameters consecutively. 
    The state variables, driver variables, process variables and parameters are
  all stored in global arrays, called stat, drive, proc, and par, respectively.
  The function counts accesses the global arrays directly but the other functions
  operate on copies of the global arrays. }
unit equations;

interface

uses  stypes, math;

PROCEDURE counts;
PROCEDURE processes(time:double; var tdrive:drivearray; var tpar:paramarray;
                       var tstat:statearray;var tproc:processarray;
                       CalculateDiscrete:Boolean);
PROCEDURE derivs(t:double; var tdrive:drivearray; var tpar:paramarray;
             var statevalue:yValueArray; VAR dydt:yValueArray);
function ParCount(processnum:integer) : integer;

var
  tproc:processarray;
  tstat:statearray;
  sensflag:boolean;
  newyear:Boolean = false;

implementation

uses frontend, calculate;

           { Do not make modifcations above this line. }
{*****************************************************************************}

{ This procedure defines the model. The number of parameters, state, driver and
  process variables are all set in this procedure. The model name, version
  number and time unit are also set here. This procedure accesses the global
  arrays containing the the parameters, state, driver and process variables and
  the global structure ModelDef directly, to save memory space. }
PROCEDURE counts;
var
 i,npar,CurrentProc:integer;
begin
{ Set the modelname, version and time unit. }
ModelDef.modelname := 'General Ecosystem Model';
ModelDef.versionnumber := '6.4.2';
ModelDef.timeunit := 'days';

{ Set the number of state variables in the model. The maximum number of state
  variables is maxstate, in unit stypes. }
ModelDef.numstate:=55;

{ Enter the name, units and symbol for each state variable. The maximum length
  of the state variable name is 17 characters and the maximum length for units
  and symbol is 20 characters. }
 
with stat[1] do
 begin
    name:='C: foliar';  units:='g C m-2 ground';  symbol:='Cf';
 end;

with stat[2] do
 begin
    name:='C: sapwood';  units:='g C m-2 ground';  symbol:='Csw';
 end;

with stat[3] do
 begin
    name:='C: heartwood';  units:='g C m-2 ground';  symbol:='Chw';
 end;

with stat[4] do
 begin
    name:='C: root';  units:='g C m-2 ground';  symbol:='Cr';
 end;

with stat[5] do
 begin
    name:='N: foliar';  units:='g N m-2 ground';  symbol:='Nf';
 end;

with stat[6] do
 begin
    name:='N: sapwood';  units:='g N m-2 ground';  symbol:='Nsw';
 end;

with stat[7] do
 begin
    name:='N: heartwood';  units:='g N m-2 ground';  symbol:='Nhw';
 end;

with stat[8] do
 begin
    name:='N: root';  units:='g N m-2 ground';  symbol:='Nr';
 end;
 
with stat[9] do
 begin
    name:='C: labile foliar & wood';  units:='g C m-2 ground';  symbol:='Clfw';
 end;

with stat[10] do
 begin
    name:='C: labile root';  units:='g C m-2 ground';  symbol:='Clr';
 end;

with stat[11] do
 begin
    name:='N: labile foliar';  units:='g N m-2 ground';  symbol:='Nlf';
 end;

with stat[12] do
 begin
    name:='N: labile wood & root';  units:='g N m-2 ground';  symbol:='Nlwr';
 end;
 
with stat[13] do
 begin
    name:='C: acid soluble soil';  units:='g C m-2 ground';  symbol:='Cs';
 end;

with stat[14] do
 begin
    name:='C: extractive soil';  units:='g C m-2 ground';  symbol:='Ce';
 end;
 
with stat[15] do
 begin
    name:='C: acid insoluble soil';  units:='g C m-2 ground';  symbol:='Ci';
 end;

with stat[16] do
 begin
    name:='C: humus soil';  units:='g C m-2 ground';  symbol:='Ch';
 end;
 
with stat[17] do
 begin
    name:='N: active soil';  units:='g N m-2 ground';  symbol:='Na';
 end;
 
with stat[18] do
 begin
    name:='N: humus soil';  units:='g N m-2 ground';  symbol:='Nh';
 end;

with stat[19] do
 begin
    name:='NH4: total soil';  units:='g N m-2 ground';  symbol:='Nnh4';
 end;

with stat[20] do
 begin
    name:='NO3: total soil';  units:='g N m-2 ground';  symbol:='Nno3';
 end;

with stat[21] do
 begin
    name:='H2O: Root-zone';  units:='mm H2O';  symbol:='Wr';
 end;

with stat[22] do
 begin
    name:='H2O: Groundwater';  units:='mm H2O';  symbol:='Wg';
 end;

with stat[23] do
 begin
    name:='H2O: Snowpack';  units:='mm H2O';  symbol:='Ws';
 end;

with stat[24] do
 begin
    name:='LAI: End of Grow season';  units:='m2 leaf m-2 grd';  symbol:='Laieos';
 end;

with stat[25] do
 begin
    name:='Julian day';  units:='day';  symbol:='Jday';
 end;

with stat[26] do
 begin
    name:='degree days, air';  units:='degree C day';  symbol:='DDa';
 end;

with stat[27] do
 begin
    name:='degree days, soil';  units:='degree C day';  symbol:='DDs';
 end;

with stat[28] do
 begin
    name:='N Uptake mean cost';  units:='g C g-1 N';  symbol:='qUnT';
 end;

with stat[29] do
 begin
    name:='Cum. Precipitation';  units:='mm H2O yr-1';  symbol:='PptT';
 end;

with stat[30] do
 begin
    name:='Cum. Interception';  units:='mm H2O yr-1';  symbol:='PptIT';
 end;

with stat[31] do
 begin
    name:='Cum. Evapotranspiration';  units:='mm H2O yr-1';  symbol:='EtT';
 end;

with stat[32] do
 begin
    name:='Cum. Discharge';  units:='mm H2O yr-1';  symbol:='QgT';
 end;

with stat[33] do
 begin
    name:='Cum. Water Losses';  units:='mm H2O yr-1';  symbol:='QwT';
 end;

with stat[34] do
 begin
    name:='Cum. N Deposition';  units:='g N m-2 yr-1';  symbol:='DnT';
 end;

with stat[35] do
 begin
    name:='Cum. GPP';  units:='g C m-2 yr-1';  symbol:='PST';
 end;

with stat[36] do
 begin
    name:='Cum. NPP';  units:='g C m-2 yr-1';  symbol:='NPPT';
 end;

with stat[37] do
 begin
    name:='Cum. translocation';  units:='g N m-2 yr-1';  symbol:='tauNT';
 end;

with stat[38] do
 begin
    name:='Cum. resp. plant';  units:='g C m-2 yr-1';  symbol:='RPT';
 end;

with stat[39] do
 begin
    name:='Cum. litter foliar C';  units:='g C m-2 yr-1';  symbol:='lambdaCfT';
 end;

with stat[40] do
 begin
    name:='Cum. litter wood C';  units:='g C m-2 yr-1';  symbol:='lambdaCwT';
 end;

with stat[41] do
 begin
    name:='Cum. litter root C';  units:='g C m-2 yr-1';  symbol:='lambdaCrT';
 end;

with stat[42] do
 begin
    name:='Cum. litter foliar N';  units:='g N m-2 yr-1';  symbol:='lambdaNfT';
 end;
 
with stat[43] do
 begin
    name:='Cum. litter wood N';  units:='g N m-2 yr-1';  symbol:='lambdaNwT';
 end;

with stat[44] do
 begin
    name:='Cum. litter root N';  units:='g N m-2 yr-1';  symbol:='lambdaNrT';
 end;

with stat[45] do
 begin
    name:='Cum. litter C';  units:='g C m-2 yr-1';  symbol:='lambdaCT';
 end;

with stat[46] do
 begin
    name:='Cum. litter N';  units:='g N m-2 yr-1';  symbol:='lambdaNT';
 end;

with stat[47] do
 begin
    name:='Cum. resp. soil';  units:='g C m-2 yr-1';  symbol:='RST';
 end;

with stat[48] do
 begin
    name:='Cum. NEP';  units:='g C m-2 yr-1';  symbol:='NepT';
 end;

with stat[49] do
 begin
    name:='Cum. N uptake';  units:='g N m-2 yr-1';  symbol:='UNT';
 end;

with stat[50] do
 begin
    name:='Cum. Nfix';  units:='g N m-2 yr-1';  symbol:='UnfixT';
 end;

with stat[51] do
 begin
    name:='Cum. Net N min';  units:='g N m-2 yr-1';  symbol:='NminT';
 end;

with stat[52] do
 begin
    name:='Cum. N leaching';  units:='g N m-2 yr-1';  symbol:='QNT';
 end;

with stat[53] do
 begin
    name:='Jday Fol Sens = 999';  units:='Julian Day';  symbol:='Jdfs';
 end;

with stat[54] do
 begin
    name:='Jday Wood Sens = 999';  units:='Julian Day';  symbol:='Jdws';
 end;

with stat[55] do
 begin
    name:='Jday Root Sens = 999';  units:='Julian Day';  symbol:='Jdrs';
 end;


{ Set the total number of processes in the model. The first numstate processes
  are the derivatives of the state variables. The maximum number of processes is
  maxparam, in unit stypes. }
ModelDef.numprocess := ModelDef.numstate + 134;

{ For each process, set proc[i].parameters equal to the number of parameters
  associated with that process, and set IsDiscrete to true or false. After each
  process, set the name, units, and symbol for all parameters associated with
  that process. Note that Parcount returns the total number of parameters in
  all previous processes. }


CurrentProc := ModelDef.numstate + 1;
With proc[CurrentProc] do
   begin
      name       := 'Temp: air mean daily';
      units       := 'oC';
      symbol       := 'Ta';
      parameters       := 0;
      ptype        := ptMisc;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 2;
With proc[CurrentProc] do
   begin
      name       := 'Temp: soil mean daily';
      units       := 'oC';
      symbol       := 'Ts';
      parameters       := 0;
      ptype        := ptMisc;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 3;
With proc[CurrentProc] do
   begin
      name       := 'Day length';
      units       := 'hours';
      symbol       := 'Hd';
      parameters       := 1;
      ptype        := ptMisc;
      IsDiscrete       := True;
   end;
npar:=ParCount(CurrentProc);
with par[npar+1] do
 begin
    name:='latitude';  units:='degrees';  symbol:='Deltad';
 end;

CurrentProc := ModelDef.numstate + 4;
With proc[CurrentProc] do
   begin
      name       := 'H2O: Rainfall';
      units       := 'mm H2O day-1';
      symbol       := 'Pptr';
      parameters       := 1;
      ptype        := ptWater;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar+1] do
 begin
    name:='critical Ta for snowfall';  units:='oC';  symbol:='Tas';
 end;

CurrentProc := ModelDef.numstate + 5;
With proc[CurrentProc] do
   begin
      name       := 'H2O: Snowfall';
      units       := 'mm H2O day-1';
      symbol       := 'Ppts';
      parameters       := 0;
      ptype        := ptWater;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 6;
With proc[CurrentProc] do
   begin
      name       := 'H2O: Interception';
      units       := 'mm H2O day-1';
      symbol       := 'PptI';
      parameters       := 2;
      ptype        := ptWater;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar+1] do
 begin
    name:='intercept/LAI';  units:='mmH2O m2 gnd m-2 leaf day-1';  symbol:='bIf';
 end;
with par[npar+2] do
 begin
    name:='intercept/sapwood';  units:='mm H2O m2 g-1 C day-1';  symbol:='bIsw';
 end;

CurrentProc := ModelDef.numstate + 7;
With proc[CurrentProc] do
   begin
      name       := 'H2O: Available';
      units       := 'fraction';
      symbol       := 'Aw';
      parameters       := 2;
      ptype        := ptWater;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar+1] do
 begin
    name:='soil H2O @ wilting';  units:='mm H2O';  symbol:='Ww';
 end;
with par[npar+2] do
 begin
    name:='soil H2O @ field cap';  units:='mm H2O';  symbol:='Wf';
 end;

CurrentProc := ModelDef.numstate + 8;
With proc[CurrentProc] do
   begin
      name       := 'H2O: tension deficit';
      units       := 'MPa';
      symbol       := 'PsiD';
      parameters       := 2;
      ptype        := ptWater;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar+1] do
 begin
    name:='tension @ wilting';  units:='MPa';  symbol:='Psiw';
 end;
with par[npar+2] do
 begin
    name:='tension @ field cap';  units:='MPa';  symbol:='Psif';
 end;

CurrentProc := ModelDef.numstate + 9;
With proc[CurrentProc] do
   begin
      name       := 'H2O: Canopy conductance';
      units       := 'm H2O sec-1';
      symbol       := 'gsw';
      parameters       := 0;
      ptype        := ptWater;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 10;
With proc[CurrentProc] do
   begin
      name       := 'H2O: Evapotranspiration';
      units       := 'mm H2O day-1';
      symbol       := 'ET';
      parameters       := 2;
      ptype        := ptWater;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar+1] do
 begin
    name:='canopy albedo';  units:='unitless';  symbol:='alphaT';
 end;
with par[npar+2] do
 begin
    name:='boundary H2O cond';  units:='m H2O sec-1';  symbol:='gAW';
 end;

CurrentProc := ModelDef.numstate + 11;
With proc[CurrentProc] do
   begin
      name       := 'H2O: Snowmelt';
      units       := 'mm H2O day-1';
      symbol       := 'Ms';
      parameters       := 3;
      ptype        := ptWater;
      IsDiscrete       := True;
   end;
npar:=ParCount(CurrentProc);
with par[npar+1] do
 begin
    name:='forest canopy denst';  units:='fraction';  symbol:='rhot';
 end;
with par[npar+2] do
 begin
    name:='snow albedo';  units:='fraction';  symbol:='alphas';
 end;
with par[npar+3] do
 begin
    name:='cover corr. factor';  units:='unitless';  symbol:='gammat';
 end;

CurrentProc := ModelDef.numstate + 12;
With proc[CurrentProc] do
   begin
      name       := 'H2O: soil drainage';
      units       := 'mm H2O day-1';
      symbol       := 'DR';
      parameters       := 1;
      ptype        := ptWater;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar+1] do
 begin
    name:='drain rate';  units:='day-1';  symbol:='alphaD';
 end;

CurrentProc := ModelDef.numstate + 13;
With proc[CurrentProc] do
   begin
      name       := 'H2O: grnd discharge';
      units       := 'mm H2O day-1';
      symbol       := 'Qg';
      parameters       := 1;
      ptype        := ptWater;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar+1] do
 begin
    name:='discharge rate';  units:='day-1';  symbol:='alphaQ';
 end;
 
CurrentProc := ModelDef.numstate + 14;
With proc[CurrentProc] do
   begin
      name       := 'NH4: Aqueous';
      units       := 'umol N L-1 H2O';
      symbol       := 'Nnh4aq';
      parameters       := 4;
      ptype        := ptWater;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar+1] do
 begin
    name:='soil bulk density';  units:='g m-3';  symbol:='rhoS';
 end;
with par[npar+2] do
 begin
    name:='soil depth';  units:='m';  symbol:='z';
 end;
with par[npar+3] do
 begin
    name:='max NH4 sorbed';  units:='g N g-1 dry soil';  symbol:='Vnh4max';
 end;
with par[npar+4] do
 begin
    name:='soil affinity NH4';  units:='umol N L-1 H2O';  symbol:='etaNH4';
 end;

CurrentProc := ModelDef.numstate + 15;
With proc[CurrentProc] do
   begin
      name       := 'NO3: Aqueous';
      units       := 'umol N L-1 H2O';
      symbol       := 'Nno3aq';
      parameters       := 2;
      ptype        := ptWater;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar+1] do
 begin
    name:='max NO3 sorbed';  units:='g N g-1 dry soil';  symbol:='Vno3max';
 end;
with par[npar+2] do
 begin
    name:='soil affinity NO3';  units:='umol N L-1 H2O';  symbol:='etaNO3';
 end;

CurrentProc := ModelDef.numstate + 16;
With proc[CurrentProc] do
   begin
      name       := 'NH4: sorbed';
      units       := 'g N m-2 ground';
      symbol       := 'Nnh4s';
      parameters       := 0;
      ptype        := ptWater;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 17;
With proc[CurrentProc] do
   begin
      name       := 'NO3: sorbed';
      units       := 'g N m-2 ground';
      symbol       := 'Nno3s';
      parameters       := 0;
      ptype        := ptWater;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 18;
With proc[CurrentProc] do
   begin
      name       := 'NH4: loss in water';
      units       := 'g N m-2 day-1';
      symbol       := 'QNH4';
      parameters       := 0;
      ptype        := ptWater;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 19;
With proc[CurrentProc] do
   begin
      name       := 'NO3: loss in water';
      units       := 'g N m-2 day-1';
      symbol       := 'QNO3';
      parameters       := 0;
      ptype        := ptWater;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 20;
With proc[CurrentProc] do
   begin
      name       := 'C: Sapwood maximum';
      units       := 'g C m-2';
      symbol       := 'Cswmax';
      parameters       := 2;
      ptype        := ptVegetation;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar+1] do
 begin
    name:='sap wood coefficient'; units:='empirical relationship'; symbol:='h1';
 end;
with par[npar+2] do
 begin
    name:='sap wood exponent'; units:='empirical relationship'; symbol:='h2';
 end;

CurrentProc := ModelDef.numstate + 21;
With proc[CurrentProc] do
   begin
      name       := 'Canopy C conductance';
      units       := 'g C m-2 ppm-1 day-1';
      symbol       := 'gsC';
      parameters       := 1;
      ptype        := ptVegetation;
      IsDiscrete       := True;
   end;
npar:=ParCount(CurrentProc);
with par[npar+1] do
 begin
    name:='soil hydraulic res';  units:='sec m2 MPa mmol-1';  symbol:='omegaS';
 end;

CurrentProc := ModelDef.numstate + 22;
With proc[CurrentProc] do
   begin
      name       := 'Leaf Area Index';
      units       := 'm2 leaf m-2 ground';
      symbol       := 'LAI';
      parameters       := 1;
      ptype        := ptVegetation;
      IsDiscrete       := True;
   end;
npar:=ParCount(CurrentProc);
with par[npar+1] do
 begin
    name:='Foliage C Density';  units:='g C m-2 leaf';  symbol:='mF';
 end;

CurrentProc := ModelDef.numstate + 23;
With proc[CurrentProc] do
   begin
      name       := 'GPP: daily';
      units       := 'g C m-2 day-1';
      symbol       := 'Ps';
      parameters       := 3;
      ptype        := ptVegetation;
      IsDiscrete       := True;
   end;
npar:=ParCount(CurrentProc);
with par[npar+1] do
 begin
    name:='leaf N Ps correction';  units:='unitless';  symbol:='aNf';
 end;
with par[npar+2] do
 begin
    name:='Maximum Clfw';  units:='g C m-2';  symbol:='Clfwmax';
 end;
with par[npar+3] do
 begin
    name:='Ps sens to carbohydrate';  units:='unitless';  symbol:='epsilsonPs';
 end;

CurrentProc := ModelDef.numstate + 24;
With proc[CurrentProc] do
   begin
      name       := 'GPP: isotopic signature';
      units       := 'o/oo';
      symbol       := 'deltaPs';
      parameters       := 3;
      ptype        := ptVegetation;
      IsDiscrete       := True;
   end;
npar:=ParCount(CurrentProc);
with par[npar+1] do
 begin
    name:='isotopic signature of atm';  units:='o/oo';  symbol:='deltaCa';
 end;
with par[npar+2] do
 begin
    name:='diffusion fractionation';  units:='o/oo';  symbol:='aDelta';
 end;
with par[npar+3] do
 begin
    name:='net carboxylation fract';  units:='o/oo';  symbol:='bDelta';
 end;

CurrentProc := ModelDef.numstate + 25;
With proc[CurrentProc] do
   begin
      name       := 'N: Drought Transloc.';
      units       := 'g N m-2 day-1';
      symbol       := 'tauND';
      parameters       := 0;    // Parameter is listed with drought litterfall.
      ptype        := ptVegetation;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 26;
With proc[CurrentProc] do
   begin
      name       := 'N: translocation';
      units       := 'g N m-2 day-1';
      symbol       := 'tauN';
      parameters       := 1;
      ptype        := ptVegetation;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar+1] do
 begin
    name:='translocation fraction';  units:='fraction';  symbol:='atau';
 end;

CurrentProc := ModelDef.numstate + 27;
With proc[CurrentProc] do
   begin
      name       := 'Uptake: temp response';
      units       := 'unitless';
      symbol       := 'HU';
      parameters       := 3;
      ptype        := ptVegetation;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar+1] do
 begin
    name:='Uptake max temp';  units:='oC';  symbol:='TUmax';
 end;
with par[npar+2] do
 begin
    name:='Uptake opt temp';  units:='oC';  symbol:='TUopt';
 end;
with par[npar+3] do
 begin
    name:='Uptake T curvature';  units:='oC-1';  symbol:='gammaU';
 end;

CurrentProc := ModelDef.numstate + 28;
With proc[CurrentProc] do
   begin
      name       := 'NH4: uptake';
      units       := 'g N m-2 day-1';
      symbol       := 'UNH4';
      parameters       := 5;
      ptype        := ptVegetation;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar+1] do
 begin
    name:='max NH4 uptake'; units:='g N m-2 day-1'; symbol:='UNH4max';
 end;
with par[npar+2] do
 begin
    name:='NH4 1/2 saturat';  units:='umol N L-1 H2O';  symbol:='kNH4';
 end;
with par[npar+3] do
 begin
    name:='Un Sens. to Nr';  units:='m2 gnd g-1 C';  symbol:='thetaU';
 end;
with par[npar+4] do
 begin
    name:='Maximum Nlwr';  units:='g N m-2';  symbol:='Nlwrmax';
 end;
with par[npar+5] do
 begin
    name:='Un sens to labile N';  units:='unitless';  symbol:='epsilsonUn';
 end;

CurrentProc := ModelDef.numstate + 29;
With proc[CurrentProc] do
   begin
      name       := 'NO3: uptake';
      units       := 'g N m-2 day-1';
      symbol       := 'UNO3';
      parameters       := 2;
      ptype        := ptVegetation;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar+1] do
 begin
    name:='max NO3 uptake'; units:='g N m-2 day-1'; symbol:='UNO3max';
 end;
with par[npar+2] do
 begin
    name:='NO3 1/2 saturat';  units:='umol N L-1 H2O';  symbol:='kNO3';
 end;

CurrentProc := ModelDef.numstate + 30;
With proc[CurrentProc] do
   begin
      name       := 'N Uptake cost - daily';
      units       := 'g C g-1 N';
      symbol       := 'qUn';
      parameters       := 1;
      ptype        := ptVegetation;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar+1] do
 begin
    name:='N uptake cost coeff'; units:='gC g-1 N MJ-1';  symbol:='gUn';
 end;

CurrentProc := ModelDef.numstate + 31;
With proc[CurrentProc] do
   begin
      name       := 'N: Fixation';
      units       := 'g N m-2 day-1';
      symbol       := 'Unfix';
      parameters       := 3;
      ptype        := ptVegetation;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar+1] do
 begin
    name:='Cost of N Fixation'; units:='g C g-1 N'; symbol:='qUnfix';
 end;
with par[npar+2] do
 begin
    name:='Nfix constant';  units:='g N g-1 Cr day-1';  symbol:='gNfix';
 end;
with par[npar+3] do
 begin
    name:='kNfix';  units:='g C g-1 N';  symbol:='kNfix';
 end;

CurrentProc := ModelDef.numstate + 32;
With proc[CurrentProc] do
   begin
      name       := 'Resp: N Fixation';
      units       := 'g C m-2 day-1';
      symbol       := 'RUnfix';
      parameters       := 0;
      ptype        := ptVegetation;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 33;
With proc[CurrentProc] do
   begin
      name       := 'Resp: temp response';
      units       := 'unitless';
      symbol       := 'HR';
      parameters       := 3;
      ptype        := ptVegetation;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar+1] do
 begin
    name:='Resp max temp';  units:='oC';  symbol:='TRmax';
 end;
with par[npar+2] do
 begin
    name:='Resp opt temp';  units:='oC';  symbol:='TRopt';
 end;
with par[npar+3] do
 begin
    name:='Resp T curvature';  units:='oC-1';  symbol:='gammaR';
 end;

CurrentProc := ModelDef.numstate + 34;
With proc[CurrentProc] do
   begin
      name       := 'Resp: Metabolic foliage';
      units       := 'g C m-2 day-1';
      symbol       := 'RFmet';
      parameters       := 2;
      ptype        := ptVegetation;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar+1] do
 begin
    name:='Met resp const';  units:='g C g-1 N day-1';  symbol:='rm';
 end;
with par[npar+2] do
 begin
    name:='Min. Fol. C/N';  units:='g C g-1 N';  symbol:='qFmin';
 end;

CurrentProc := ModelDef.numstate + 35;
With proc[CurrentProc] do
   begin
      name       := 'Resp: Metabolic wood';
      units       := 'g C m-2 day-1';
      symbol       := 'RWmet';
      parameters       := 2;
      ptype        := ptVegetation;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar+1] do
 begin
    name:='Wood resp corr factor';  units:='unitless';  symbol:='awr';
 end;
with par[npar+2] do
 begin
    name:='Min. Wood C/N';  units:='g C g-1 N';  symbol:='qWmin';
 end;

CurrentProc := ModelDef.numstate + 36;
With proc[CurrentProc] do
   begin
      name       := 'Resp: Metabolic root';
      units       := 'g C m-2 day-1';
      symbol       := 'RRmet';
      parameters       := 1;
      ptype        := ptVegetation;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar+1] do
 begin
    name:='Min. Root C/N';  units:='g C g-1 N';  symbol:='qRmin';
 end;

CurrentProc := ModelDef.numstate + 37;
With proc[CurrentProc] do
   begin
      name       := 'Resp: Growth foliage';
      units       := 'g C m-2 day-1';
      symbol       := 'RFgr';
      parameters       := 1;
      ptype        := ptVegetation;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar+1] do
 begin
    name:='growth resp coeff';  units:='fraction';  symbol:='rg';
 end;

CurrentProc := ModelDef.numstate + 38;
With proc[CurrentProc] do
   begin
      name       := 'Resp: Growth wood';
      units       := 'g C m-2 day-1';
      symbol       := 'RWgr';
      parameters       := 0;
      ptype        := ptVegetation;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 39;
With proc[CurrentProc] do
   begin
      name       := 'Resp: Growth root';
      units       := 'g C m-2 day-1';
      symbol       := 'RRgr';
      parameters       := 0;
      ptype        := ptVegetation;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 40;
With proc[CurrentProc] do
   begin
      name       := 'Resp: Labile Fol. Frac.';
      units       := 'unitless';
      symbol       := 'fRLF';
      parameters       := 1;
      ptype        := ptVegetation;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar+1] do
 begin
    name:='sharpness of eff of CLj';  units:='m2 g-1 C';  symbol:='sL';
 end;


CurrentProc := ModelDef.numstate + 41;
With proc[CurrentProc] do
   begin
      name       := 'Resp: Labile Wood Frac.';
      units       := 'unitless';
      symbol       := 'fRLW';
      parameters       := 0;
      ptype        := ptVegetation;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 42;
With proc[CurrentProc] do
   begin
      name       := 'Resp: Labile Root Frac.';
      units       := 'unitless';
      symbol       := 'fRLR';
      parameters       := 0;
      ptype        := ptVegetation;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 43;
With proc[CurrentProc] do
   begin
      name       := 'Resp: Labile Foliage';
      units       := 'g C m-2 day-1';
      symbol       := 'RfmetL';
      parameters       := 0;
      ptype        := ptVegetation;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 44;
With proc[CurrentProc] do
   begin
      name       := 'Resp: Labile Wood';
      units       := 'g C m-2 day-1';
      symbol       := 'RrmetL';
      parameters       := 0;
      ptype        := ptVegetation;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 45;
With proc[CurrentProc] do
   begin
      name       := 'Resp: Labile Root';
      units       := 'g C m-2 day-1';
      symbol       := 'RrmetL';
      parameters       := 0;
      ptype        := ptVegetation;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 46;
With proc[CurrentProc] do
   begin
      name       := 'Resp: Struct. Foliage';
      units       := 'g C m-2 day-1';
      symbol       := 'RfmetS';
      parameters       := 0;
      ptype        := ptVegetation;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 47;
With proc[CurrentProc] do
   begin
      name       := 'Resp: Struct. Wood';
      units       := 'g C m-2 day-1';
      symbol       := 'RwmetS';
      parameters       := 0;
      ptype        := ptVegetation;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 48;
With proc[CurrentProc] do
   begin
      name       := 'Resp: Struct. Root';
      units       := 'g C m-2 day-1';
      symbol       := 'RrmetS';
      parameters       := 0;
      ptype        := ptVegetation;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 49;
With proc[CurrentProc] do
   begin
      name       := 'Translocation: Resp Fol';
      units       := 'g N m-2 day-1';
      symbol       := 'tauRnf';
      parameters       := 0;
      ptype        := ptVegetation;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 50;
With proc[CurrentProc] do
   begin
      name       := 'Translocation: Resp wood';
      units       := 'g N m-2 day-1';
      symbol       := 'tauRnsw';
      parameters       := 0;
      ptype        := ptVegetation;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 51;
With proc[CurrentProc] do
   begin
      name       := 'Translocation: Resp root';
      units       := 'g N m-2 day-1';
      symbol       := 'tauRnr';
      parameters       := 0;
      ptype        := ptVegetation;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 52;
With proc[CurrentProc] do
   begin
      name       := 'Transport: C';
      units       := 'g C m-2 day-1';
      symbol       := 'Tc';
      parameters       := 2;
      ptype        := ptAllocation;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar+1] do
 begin
    name:='CL transport rate';  units:='m2 gnd m-2 leaf d-1';  symbol:='aLC';
 end;
with par[npar+2] do
 begin
    name:='CL trans. distrib.';  units:='unitless';  symbol:='bLC';
 end;

CurrentProc := ModelDef.numstate + 53;
With proc[CurrentProc] do
   begin
      name       := 'Transport: N';
      units       := 'g N m-2 day-1';
      symbol       := 'Tn';
      parameters       := 2;
      ptype       := ptAllocation;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar+1] do
 begin
    name:='NL transport rate';  units:='m2 gnd m-2 leaf d-1';  symbol:='aLN';
 end;
with par[npar+2] do
 begin
    name:='NL trans. distrib.';  units:='unitless';  symbol:='bLN';
 end;

CurrentProc := ModelDef.numstate + 54;
With proc[CurrentProc] do
   begin
      name       := 'Foliage Phenology';
      units       := 'unitless';
      symbol       := 'none';
      parameters       := 6;
      ptype        := ptMisc;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar+1] do
 begin
    name:='Day to begin DD count';  units:='Julian day';  symbol:='Jddstart';
 end;
with par[npar+2] do
 begin
    name:='Degree day fol start';  units:='oC day';  symbol:='DDfstart';
 end;
with par[npar+3] do
 begin
    name:='full canopy degree day';  units:='oC day';  symbol:='DDfstop';
 end;
with par[npar+4] do
 begin
    name:='Litter Season Start';  units:='Julian Day';  symbol:='JlambdaStart';
 end;
with par[npar+5] do
 begin
    name:='Crit Temp for Fol fall';  units:='oC';  symbol:='TacrF';
 end;
with par[npar+6] do
 begin
    name:='# days for fol senes';  units:='days';  symbol:='DsensF';
 end;

CurrentProc := ModelDef.numstate + 55;
With proc[CurrentProc] do
   begin
      name       := 'Wood Phenology';
      units       := 'unitless';
      symbol       := 'none';
      parameters       := 4;
      ptype        := ptMisc;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar+1] do
 begin
    name:='Degree day wood start';  units:='oC day';  symbol:='DDwstart';
 end;
with par[npar+2] do
 begin
    name:='Degree day wood stop';  units:='oC day';  symbol:='DDwstop';
 end;
with par[npar+3] do
 begin
    name:='Crit Temp for Wood fall';  units:='oC';  symbol:='TacrW';
 end;
with par[npar+4] do
 begin
    name:='# days for wood senes';  units:='days';  symbol:='DsensW';
 end;

CurrentProc := ModelDef.numstate + 56;
With proc[CurrentProc] do
   begin
      name       := 'Root Phenology';
      units       := 'unitless';
      symbol       := 'none';
      parameters       := 4;
      ptype        := ptMisc;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar+1] do
 begin
    name:='Degree day root start';  units:='oC day';  symbol:='DDrstart';
 end;
with par[npar+2] do
 begin
    name:='Degree day root stop';  units:='oC day';  symbol:='DDrstop';
 end;
with par[npar+3] do
 begin
    name:='Crit Temp for Root fall';  units:='oC';  symbol:='TacrR';
 end;
with par[npar+4] do
 begin
    name:='# days for Root senes';  units:='days';  symbol:='DsensR';
 end;

CurrentProc := ModelDef.numstate + 57;
With proc[CurrentProc] do
   begin
      name       := 'Foliage Growing Season';
      units       := 'unitless';
      symbol       := 'FGrow';
      parameters       := 0;
      ptype        := ptMisc;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 58;
With proc[CurrentProc] do
   begin
      name       := 'Wood Growing Season';
      units       := 'unitless';
      symbol       := 'WGrow';
      parameters       := 0;
      ptype        := ptMisc;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 59;
With proc[CurrentProc] do
   begin
      name       := 'Root Growing Season';
      units       := 'unitless';
      symbol       := 'RGrow';
      parameters       := 0;
      ptype        := ptMisc;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 60;
With proc[CurrentProc] do
   begin
      name       := 'Growth: Foliage C';
      units       := 'g C m-2 day-1';
      symbol       := 'Gcf';
      parameters       := 3;
      ptype       := ptAllocation;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar+1] do
 begin
    name:='Foliage growth rate'; units:='(g-1 C m2)^epsilonF day-1';
                                 symbol:='gf';
 end;
with par[npar+2] do
 begin
    name:='Fol. N growth parameter';  units:='(g C g-1 N)^epsilonF';  symbol:='bGf';
 end;
with par[npar+3] do
 begin
    name:='Fol. growth sensitivity';  units:='unitless';  symbol:='epsilonF';
 end;

CurrentProc := ModelDef.numstate + 61;
With proc[CurrentProc] do
   begin
      name       := 'Growth: Sapwood C';
      units       := 'g C m-2 day-1';
      symbol       := 'Gcsw';
      parameters       := 3;
      ptype       := ptAllocation;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar+1] do
 begin
    name:='Wood growth rate';  units:='(g-1 C m2)^epsilonW day-1';  symbol:='gw';
 end;
with par[npar+2] do
 begin
    name:='Wood N growth parameter';  units:='(g C g-1 N)^epsilonF';  symbol:='bGw';
 end;
with par[npar+3] do
 begin
    name:='Wood growth sensitivity';  units:='unitless';  symbol:='epsilonW';
 end;

CurrentProc := ModelDef.numstate + 62;
With proc[CurrentProc] do
   begin
      name       := 'Growth: Heartwood C';
      units       := 'g C m-2 day-1';
      symbol       := 'Gchw';
      parameters       := 1;
      ptype        := ptAllocation;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar+1] do
 begin
    name:='Heartwood growth rate';  units:='day-1';  symbol:='aGHW';
 end;

CurrentProc := ModelDef.numstate + 63;
With proc[CurrentProc] do
   begin
      name       := 'Growth: Root C';
      units       := 'g C m-2 day-1';
      symbol       := 'Gcr';
      parameters       := 3;
      ptype       := ptAllocation;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar+1] do
 begin
    name:='Root growth rate'; units:='(g-1 C m2)^epsilonR day-1'; symbol:='gr';
 end;
with par[npar+2] do
 begin
    name:='Root N growth parameter'; units:='(g C g-1 N)^epsilonF'; symbol:='bGr';
 end;
with par[npar+3] do
 begin
    name:='Root growth sensitivity';  units:='unitless';  symbol:='epsilonR';
 end;

CurrentProc := ModelDef.numstate + 64;
With proc[CurrentProc] do
   begin
      name       := 'Growth: Foliage N';
      units       := 'g N m-2 day-1';
      symbol       := 'Gnf';
      parameters       := 0;
      ptype       := ptAllocation;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 65;
With proc[CurrentProc] do
   begin
      name       := 'Growth: Sapwood N';
      units       := 'g N m-2 day-1';
      symbol       := 'Gnsw';
      parameters       := 0;
      ptype       := ptAllocation;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 66;
With proc[CurrentProc] do
   begin
      name       := 'Growth: Heartwood N';
      units       := 'g N m-2 day-1';
      symbol       := 'Gnhw';
      parameters       := 0;
      ptype        := ptAllocation;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 67;
With proc[CurrentProc] do
   begin
      name       := 'Growth: Root N';
      units       := 'g N m-2 day-1';
      symbol       := 'Gnr';
      parameters       := 0;
      ptype       := ptAllocation;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 68;
With proc[CurrentProc] do
   begin
      name       := 'Drought Litter C: Foliage';
      units       := 'g C m-2 day-1';
      symbol       := 'lambdaCFD';
      parameters       := 4;
      ptype        := ptVegetation;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar+1] do
 begin
    name:='Drought fol litter rate';  units:='day-1';  symbol:='alambdaCFD';
 end;
with par[npar+2] do
 begin
    name:='Drought root litter rate';  units:='day-1';  symbol:='alambdaCRD';
 end;
with par[npar+3] do
 begin
    name:='Drought avail H2O';  units:='mm H2O';  symbol:='Awd';
 end;
with par[npar+4] do
 begin
    name:='Drought N trans rate';  units:='day-1';  symbol:='atauD';
 end;

CurrentProc := ModelDef.numstate + 69;
With proc[CurrentProc] do
   begin
      name       := 'Drought Litter C: Roots';
      units       := 'g C m-2 day-1';
      symbol       := 'lambdaCRD';
      parameters       := 0;
      ptype        := ptVegetation;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 70;
With proc[CurrentProc] do
   begin
      name       := 'Drought Litter N: Foliage';
      units      := 'g N m-2 day-1';
      symbol     := 'lambdaNFD';
      parameters := 0;
      ptype      := ptVegetation;
      IsDiscrete := False;
   end;

CurrentProc := ModelDef.numstate + 71;
With proc[CurrentProc] do
   begin
      name       := 'Drought Litter N: Roots';
      units       := 'g N m-2 day-1';
      symbol       := 'lambdaNRD';
      parameters       := 0;
      ptype        := ptVegetation;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 72;
With proc[CurrentProc] do
   begin
      name       := 'Foliage Litter Season';
      units       := 'unitless';
      symbol       := 'FFall';
      parameters       := 0;
      ptype       := ptVegetation;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 73;
With proc[CurrentProc] do
   begin
      name       := 'Wood Litter Season';
      units       := 'unitless';
      symbol       := 'WFall';
      parameters       := 0;
      ptype       := ptVegetation;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 74;
With proc[CurrentProc] do
   begin
      name       := 'Root Litter Season';
      units       := 'unitless';
      symbol       := 'RFall';
      parameters       := 0;
      ptype       := ptVegetation;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 75;
With proc[CurrentProc] do
   begin
      name       := 'Litter: Foliage C';
      units       := 'g C m-2 day-1';
      symbol       := 'lambdaCF';
      parameters       := 3;
      ptype        := ptVegetation;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar+1] do
 begin
    name:='F litter rate'; units:='(gC m-2)^(1-sigmaLF) dy-1'; symbol:='alambdaF';
 end;
with par[npar+2] do
 begin
    name:='F Litter exp';  units:='unitless';  symbol:='sigmaLambdaF';
 end;
with par[npar+3] do
 begin
    name:='Evergreen foliage';  units:='g C m-2';  symbol:='Cfe';
 end;

CurrentProc := ModelDef.numstate + 76;
With proc[CurrentProc] do
   begin
      name       := 'Litter: Sapwood C';
      units       := 'g C m-2 day-1';
      symbol       := 'lambdaCSW';
      parameters       := 3;
      ptype        := ptVegetation;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar+1] do
 begin
    name:='W litter rate'; units:='(gC m-2)^(1-sigmaLW) dy-1'; symbol:='alambdaW';
 end;
with par[npar+2] do
 begin
    name:='W litter exp';  units:='unitless';  symbol:='sigmaLambdaW';
 end;
with par[npar+3] do
 begin
    name:='Evergreen wood';  units:='g C m-2';  symbol:='Cwe';
 end;

CurrentProc := ModelDef.numstate + 77;
With proc[CurrentProc] do
   begin
      name       := 'Litter: Heartwood C';
      units       := 'g C m-2 day-1';
      symbol       := 'lambdaCHW';
      parameters       := 0;
      ptype        := ptVegetation;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 78;
With proc[CurrentProc] do
   begin
      name       := 'Litter: Root C';
      units       := 'g C m-2 day-1';
      symbol       := 'lambdaCR';
      parameters       := 3;
      ptype        := ptVegetation;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar+1] do
 begin
    name:='R litter rate'; units:='(gC m-2)^(1-sigmaLR) dy-1'; symbol:='alambdaR';
 end;
with par[npar+2] do
 begin
    name:='R litter exp';  units:='unitless';  symbol:='sigmaLambdaR';
 end;
with par[npar+3] do
 begin
    name:='Evergreen roots';  units:='g C m-2';  symbol:='Cre';
 end;

CurrentProc := ModelDef.numstate + 79;
With proc[CurrentProc] do
   begin
      name       := 'Litter: Foliage N';
      units       := 'g N m-2 day-1';
      symbol       := 'lambdaNF';
      parameters       := 0;
      ptype        := ptVegetation;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 80;
With proc[CurrentProc] do
   begin
      name       := 'Litter: Sapwood N';
      units       := 'g N m-2 day-1';
      symbol       := 'lambdaNSW';
      parameters       := 0;
      ptype        := ptVegetation;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 81;
With proc[CurrentProc] do
   begin
      name       := 'Litter: Heartwood N';
      units       := 'g N m-2 day-1';
      symbol       := 'lambdaNHW';
      parameters       := 0;
      ptype        := ptVegetation;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 82;
With proc[CurrentProc] do
   begin
      name       := 'Litter: Root N';
      units       := 'g N m-2 day-1';
      symbol       := 'lambdaNR';
      parameters       := 0;
      ptype        := ptVegetation;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 83;
With proc[CurrentProc] do
   begin
      name       := 'C input to extractives';
      units       := 'g C m-2 day-1';
      symbol       := 'FClambdaE';
      parameters       := 0;
      ptype        := ptSoil;
      IsDiscrete       := True;
   end;

CurrentProc := ModelDef.numstate + 84;
With proc[CurrentProc] do
   begin
      name       := 'C input to acid soluble';
      units       := 'g C m-2 day-1';
      symbol       := 'FClambdaS';
      parameters       := 0;
      ptype        := ptSoil;
      IsDiscrete       := True;
   end;

CurrentProc := ModelDef.numstate + 85;
With proc[CurrentProc] do
   begin
      name       := 'C input to acid insoluble';
      units       := 'g C m-2 day-1';
      symbol       := 'FClambdaI';
      parameters       := 0;
      ptype        := ptSoil;
      IsDiscrete       := True;
   end;

CurrentProc := ModelDef.numstate + 86;
With proc[CurrentProc] do
   begin
      name       := 'N input to active N';
      units       := 'g N m-2 day-1';
      symbol       := 'FNlambdaA';
      parameters       := 0;
      ptype        := ptSoil;
      IsDiscrete       := True;
   end;

CurrentProc := ModelDef.numstate + 87;
With proc[CurrentProc] do
   begin
      name       := 'Soil metabolic rate';
      units       := 'unitless';
      symbol       := 'rSm';
      parameters       := 4;
      ptype        := ptSoil;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar+1] do
 begin
    name:='Q10 for decomp';  units:='unitless';  symbol:='Q10D';
 end;
with par[npar+2] do
 begin
    name:='Decomp H2O skewness';  units:='unitless';  symbol:='sigmaD';
 end;
with par[npar+3] do
 begin
    name:='optimum available water';  units:='unitless';  symbol:='Awdopt';
 end;
with par[npar+4] do
 begin
    name:='Decomp H2O breadth';  units:='unitless';  symbol:='Jdb';
 end;

CurrentProc := ModelDef.numstate + 88;
With proc[CurrentProc] do
   begin
      name       := 'Soil C Sol-Extr flux';
      units       := 'g C m-2 day-1';
      symbol       := 'FCSE';
      parameters       := 1;
      ptype        := ptSoil;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar+1] do
 begin
    name:='Fcse rate const';  units:='day-1';  symbol:='aSE';
 end;

CurrentProc := ModelDef.numstate + 89;
With proc[CurrentProc] do
   begin
      name       := 'Soil C Extr-Insol flux';
      units       := 'g C m-2 day-1';
      symbol       := 'FCEI';
      parameters       := 1;
      ptype        := ptSoil;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar+1] do
 begin
    name:='Fcei rate const';  units:='day-1';  symbol:='aEI';
 end;

CurrentProc := ModelDef.numstate + 90;
With proc[CurrentProc] do
   begin
      name       := 'Soil C Insol-Extr flux';
      units       := 'g C m-2 day-1';
      symbol       := 'FCIE';
      parameters       := 1;
      ptype        := ptSoil;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar+1] do
 begin
    name:='Fcie rate const';  units:='day-1';  symbol:='aIE';
 end;

CurrentProc := ModelDef.numstate + 91;
With proc[CurrentProc] do
   begin
      name       := 'Soil C Insol-Humus flux';
      units       := 'g C m-2 day-1';
      symbol       := 'FCIH';
      parameters       := 1;
      ptype        := ptSoil;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar+1] do
 begin
    name:='Fcih rate const';  units:='day-1';  symbol:='aIH';
 end;

CurrentProc := ModelDef.numstate + 92;
With proc[CurrentProc] do
   begin
      name       := 'Soil resp active';
      units       := 'g C m-2 day-1';
      symbol       := 'RE';
      parameters       := 1;
      ptype        := ptSoil;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar+1] do
 begin
    name:='resp efficiency';  units:='fraction';  symbol:='beta';
 end;

CurrentProc := ModelDef.numstate + 93;
With proc[CurrentProc] do
   begin
      name       := 'Soil resp humus';
      units       := 'g C m-2 day-1';
      symbol       := 'RH';
      parameters       := 1;
      ptype        := ptSoil;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar+1] do
 begin
    name:='Humus resp rate';  units:='day-1';  symbol:='aH';
 end;

CurrentProc := ModelDef.numstate + 94;
With proc[CurrentProc] do
   begin
      name       := 'N Min: gross active daily';
      units       := 'g N m-2 day-1';
      symbol       := 'MNA';
      parameters       := 1;
      ptype        := ptSoil;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar+1] do
 begin
    name:='Na mineralization rate';  units:='day-1';  symbol:='aA';
 end;

CurrentProc := ModelDef.numstate + 95;
With proc[CurrentProc] do
   begin
      name       := 'N Min: gross humus daily';
      units       := 'g N m-2 day-1';
      symbol       := 'MNH';
      parameters       := 0;
      ptype        := ptSoil;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 96;
With proc[CurrentProc] do
   begin
      name       := 'Soil N act-humus flux';
      units       := 'g N m-2 day-1';
      symbol       := 'FNAH';
      parameters       := 1;
      ptype        := ptSoil;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar+1] do
 begin
    name:='opt humus C:N ratio';  units:='g C g-1 N';  symbol:='qI';
 end;

CurrentProc := ModelDef.numstate + 97;
With proc[CurrentProc] do
   begin
      name       := 'NH4: immobilization';
      units       := 'g N m-2 day-1';
      symbol       := 'INH4';
      parameters       := 2;
      ptype        := ptSoil;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar+1] do
 begin
    name:='max NH4 imm rate';  units:='day-1';  symbol:='VINH4max';
 end;
with par[npar+2] do
 begin
    name:='1/2 sat for NH4 imm';  units:='umol N L-1 H2O';  symbol:='kINH4';
 end;

CurrentProc := ModelDef.numstate + 98;
With proc[CurrentProc] do
   begin
      name       := 'NO3: immobilization';
      units       := 'g N m-2 day-1';
      symbol       := 'INO3';
      parameters       := 2;
      ptype        := ptSoil;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar+1] do
 begin
    name:='max NO3 imm rate';  units:='day-1';  symbol:='VINO3max';
 end;
with par[npar+2] do
 begin
    name:='1/2 sat for NO3 imm';  units:='umol N L-1 H2O';  symbol:='kINO3';
 end;

CurrentProc := ModelDef.numstate + 99;
With proc[CurrentProc] do
   begin
      name       := 'Nitrification';
      units       := 'g N m-2 day-1';
      symbol       := 'Fntri';
      parameters       := 1;
      ptype        := ptSoil;
      IsDiscrete       := False;
   end;
npar:=ParCount(CurrentProc);
with par[npar+1] do
 begin
    name:='nitr. rate';  units:='gN LH2O umol-1N m-2 day-1';  symbol:='aNtri';
 end;

CurrentProc := ModelDef.numstate + 100;
With proc[CurrentProc] do
   begin
      name       := 'C: Total Wood';
      units       := 'g C m-2 ground';
      symbol       := 'Cw';
      parameters       := 0;
      ptype        := ptVegetation;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 101;
With proc[CurrentProc] do
   begin
      name       := 'N: Total Wood';
      units       := 'g N m-2 ground';
      symbol       := 'Nw';
      parameters       := 0;
      ptype        := ptVegetation;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 102;
With proc[CurrentProc] do
   begin
      name       := 'C:N; Foliage';
      units       := 'g C g-1 N';
      symbol       := 'qF';
      parameters       := 0;
      ptype        := ptVegetation;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 103;
With proc[CurrentProc] do
   begin
      name       := 'C:N; Sapwood';
      units       := 'g C g-1 N';
      symbol       := 'qSW';
      parameters       := 0;
      ptype        := ptVegetation;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 104;
With proc[CurrentProc] do
   begin
      name       := 'C:N; Heartwood';
      units       := 'g C g-1 N';
      symbol       := 'qHW';
      parameters       := 0;
      ptype        := ptVegetation;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 105;
With proc[CurrentProc] do
   begin
      name       := 'C:N; Total wood';
      units       := 'g C g-1 N';
      symbol       := 'qW';
      parameters       := 0;
      ptype        := ptVegetation;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 106;
With proc[CurrentProc] do
   begin
      name       := 'C:N; Root';
      units       := 'g C g-1 N';
      symbol       := 'qR';
      parameters       := 0;
      ptype        := ptVegetation;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 107;
With proc[CurrentProc] do
   begin
      name       := 'Root:Shoot ratio';
      units       := 'unitless';
      symbol       := 'qrs';
      parameters       := 0;
      ptype        := ptSoil;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 108;
With proc[CurrentProc] do
   begin
      name       := 'C: total plant';
      units       := 'g C m-2 ground';
      symbol       := 'CPT';
      parameters       := 0;
      ptype        := ptVegetation;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 109;
With proc[CurrentProc] do
   begin
      name       := 'C: total organic soil';
      units       := 'g C m-2 ground';
      symbol       := 'CST';
      parameters       := 0;
      ptype        := ptSoil;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 110;
With proc[CurrentProc] do
   begin
      name       := 'N: total plant';
      units       := 'g N m-2 ground';
      symbol       := 'NPT';
      parameters       := 0;
      ptype        := ptVegetation;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 111;
With proc[CurrentProc] do
   begin
      name       := 'N: total organic soil';
      units       := 'g N m-2 ground';
      symbol       := 'NST';
      parameters       := 0;
      ptype        := ptSoil;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 112;
With proc[CurrentProc] do
   begin
      name       := 'C:N; total plant';
      units       := 'g C g-1 N';
      symbol       := 'qPT';
      parameters       := 0;
      ptype        := ptVegetation;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 113;
With proc[CurrentProc] do
   begin
      name       := 'C:N; total soil';
      units       := 'g C g-1 N';
      symbol       := 'qST';
      parameters       := 0;
      ptype        := ptSoil;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 114;
With proc[CurrentProc] do
   begin
      name       := 'N: total inorganic soil';
      units       := 'g N m-2 ground';
      symbol       := 'NIT';
      parameters       := 0;
      ptype        := ptSoil;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 115;
With proc[CurrentProc] do
   begin
      name       := 'C: total ecosystem';
      units       := 'g C m-2 ground';
      symbol       := 'CET';
      parameters       := 0;
      ptype        := ptMisc;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 116;
With proc[CurrentProc] do
   begin
      name       := 'N: total ecosystem';
      units       := 'g N m-2 ground';
      symbol       := 'NET';
      parameters       := 0;
      ptype        := ptMisc;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 117;
With proc[CurrentProc] do
   begin
      name       := 'Resp: Total foliage';
      units       := 'g C m-2 day-1';
      symbol       := 'RF';
      parameters       := 0;
      ptype        := ptVegetation;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 118;
With proc[CurrentProc] do
   begin
      name       := 'Resp: Total wood';
      units       := 'g C m-2 day-1';
      symbol       := 'RW';
      parameters       := 0;
      ptype        := ptVegetation;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 119;
With proc[CurrentProc] do
   begin
      name       := 'Resp: Total root';
      units       := 'g C m-2 day-1';
      symbol       := 'RR';
      parameters       := 0;
      ptype        := ptVegetation;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 120;
With proc[CurrentProc] do
   begin
      name       := 'Resp: Total plant';
      units       := 'g C m-2 day-1';
      symbol       := 'Rp';
      parameters       := 0;
      ptype        := ptVegetation;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 121;
With proc[CurrentProc] do
   begin
      name       := 'Resp: Total soil';
      units       := 'g C m-2 day-1';
      symbol       := 'Rs';
      parameters       := 0;
      ptype        := ptVegetation;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 122;
With proc[CurrentProc] do
   begin
      name       := 'NPP: foliar';
      units       := 'g C m-2 day-1';
      symbol       := 'NPPf';
      parameters       := 0;
      ptype        := ptVegetation;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 123;
With proc[CurrentProc] do
   begin
      name       := 'NPP: wood';
      units       := 'g C m-2 day-1';
      symbol       := 'NPPw';
      parameters       := 0;
      ptype        := ptVegetation;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 124;
With proc[CurrentProc] do
   begin
      name       := 'NPP: root';
      units       := 'g C m-2 day-1';
      symbol       := 'NPPr';
      parameters       := 0;
      ptype        := ptVegetation;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 125;
With proc[CurrentProc] do
   begin
      name       := 'NPP: daily';
      units       := 'g C m-2 day-1';
      symbol       := 'NPP';
      parameters       := 0;
      ptype        := ptVegetation;
      IsDiscrete       := True;
   end;

CurrentProc := ModelDef.numstate + 126;
With proc[CurrentProc] do
   begin
      name       := 'NEP: daily';
      units       := 'g C m-2 day-1';
      symbol       := 'NEP';
      parameters       := 0;
      ptype        := ptVegetation;
      IsDiscrete       := True;
   end;

CurrentProc := ModelDef.numstate + 127;
With proc[CurrentProc] do
   begin
      name       := 'Litter: total C';
      units       := 'g C m-2 day-1';
      symbol       := 'lambdaC';
      parameters       := 0;
      ptype        := ptVegetation;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 128;
With proc[CurrentProc] do
   begin
      name       := 'Litter: total N';
      units       := 'g N m-2 day-1';
      symbol       := 'lambdaN';
      parameters       := 0;
      ptype        := ptVegetation;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 129;
With proc[CurrentProc] do
   begin
      name       := 'N: Total Deposition';
      units       := 'g C m-2 day-1';
      symbol       := 'Dn';
      parameters       := 0;
      ptype        := ptVegetation;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 130;
With proc[CurrentProc] do
   begin
      name       := 'N Uptake, daily';
      units       := 'g N m-2 day-1';
      symbol       := 'Un';
      parameters       := 0;
      ptype        := ptVegetation;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 131;
With proc[CurrentProc] do
   begin
      name       := 'N Min: gross total daily';
      units       := 'g N m-2 day-1';
      symbol       := 'Mg';
      parameters       := 0;
      ptype        := ptSoil;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 132;
With proc[CurrentProc] do
   begin
      name       := 'Immobilization, tot daily';
      units       := 'g N m-2 day-1';
      symbol       := 'Id';
      parameters       := 0;
      ptype        := ptSoil;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 133;
With proc[CurrentProc] do
   begin
      name       := 'N Min: net total daily';
      units       := 'g N m-2 day-1';
      symbol       := 'Nmin';
      parameters       := 0;
      ptype        := ptSoil;
      IsDiscrete       := False;
   end;

CurrentProc := ModelDef.numstate + 134;
With proc[CurrentProc] do
   begin
      name       := 'N: loss in water';
      units       := 'g N m-2 day-1';
      symbol       := 'Qn';
      parameters       := 0;
      ptype        := ptSoil;
      IsDiscrete       := False;
   end;

{ Set the total number of drivers in the model. The maximum number of drivers is
  maxdrive, in unit stypes. }
ModelDef.numdrive := 18;

{ Set the names, units, and symbols of the drivers. The maximum length for the
  name, units and symbol is 20 characters. }


with drive[1] do
 begin
    name:='CO2, Atm';  units:='ppm';  symbol:='Ca';
 end;

with drive[2] do
 begin
    name:='Temperature Max';  units:='oC';  symbol:='Tamax';
 end;

with drive[3] do
 begin
    name:='Temperature Min';  units:='oC';  symbol:='Tamin';
 end;
 
with drive[4] do
 begin
    name:='Avg Soil Temp';  units:='oC';  symbol:='Ts';
 end;

with drive[5] do
 begin
    name:='H2O precip';  units:='mm H2O day-1';  symbol:='Ppt';
 end;

with drive[6] do
 begin
    name:='Incid Shortwave';  units:='MJ m-2 day-1';  symbol:='II';
 end;

with drive[7] do
 begin
    name:='NH4 deposition';  units:='g N m-2 day-1';  symbol:='DNH4';
 end;

with drive[8] do
 begin
    name:='NO3 deposition';  units:='g N m-2 day-1';  symbol:='DNO3';
 end;

with drive[9] do
 begin
    name:='Driver: Fol Litter C';  units:='g C m-2 day-1';  symbol:='DlambdaCF';
 end;

with drive[10] do
 begin
    name:='Driver: Sapwood Litter C'; units:='g C m-2 day-1';
                                                           symbol:='DlambdaCSW';
 end;

with drive[11] do
 begin
    name:='Driver: Heartwood Litter C'; units:='g C m-2 day-1';
                                                           symbol:='DlambdaCHW';
 end;

with drive[12] do
 begin
    name:='Driver: Root Litter C'; units:='g C m-2 day-1';  symbol:='DlambdaCR';
 end;

with drive[13] do
 begin
    name:='Driver: Fol Litter N';  units:='g N m-2 day-1';  symbol:='DlambdaNF';
 end;

with drive[14] do
 begin
    name:='Driver: Sapwood Litter N'; units:='g N m-2 day-1';
                                                           symbol:='DlambdaNSW';
 end;

with drive[15] do
 begin
    name:='Driver: Heartwood Litter N'; units:='g N m-2 day-1';
                                                           symbol:='DlambdaNHW';
 end;

with drive[16] do
 begin
    name:='Driver: Root Litter N'; units:='g N m-2 day-1';  symbol:='DlambdaNR';
 end;

with drive[17] do
 begin
    name:='Driver: Soil NH4';  units:='g N m-2 ';  symbol:='DNnh4';
 end;

with drive[18] do
 begin
    name:='Driver: Soil NO3';  units:='g N m-2';  symbol:='DNno3';
 end;


{ Set the names, units, and symbols of the processes and the parameters. The
  maximum length for the name, units and symbol is 20 characters.

  The first numstate processes are the derivatives of the state variables. Set
  the names and units accordingly.}
for i:= 1 to ModelDef.numstate do proc[i].name:='d'+stat[i].name+'dt';
for i:= 1 to ModelDef.numstate do proc[i].units := stat[i].units + 't-1';
for i:= 1 to ModelDef.numstate do proc[i].symbol := 'd' + stat[i].symbol + 'dt';

{ Sum up the total number of parameters in the model and store it in the
  ModelDef structure. }
ModelDef.numparam := 0;
for i := 1 to ModelDef.NumProcess do
  ModelDef.numparam := ModelDef.numparam + proc[i].parameters;

end; // counts procedure


{ A procedure to calculate the value of all states and processes at the current
  time. This function accesses time, state variables and process variables by
  reference, ie it uses the same array as the calling routine. It does not use
  the global variables time, stat and proc because values calculated during
  integration might later be discarded. It does access the global variables par,
  drive and ModelDef directly because those values are not modified.

  The model equations are written using variable names which correspond to the
  actual name instead of using the global arrays (i.e. SoilWater instead of
  stat[7].value). This makes it necessary to switch all values into local
  variables, do all the calculations and then put everything back into the
  global variables. Lengthy but worth it in terms of readability of the code. }

PROCEDURE processes(time:double; var tdrive:drivearray; var tpar:paramarray;
                       var tstat:statearray;var tproc:processarray;
                       CalculateDiscrete:Boolean);
const 
// Global Ps parameters
 x1 = 11.04;             x2 = 0.03;
 x5 = 0.216;             x6 = 0.6;
 x7 = 3.332;             x8 = 0.004;
 x9 = 1.549;             x10 = 1.156;
 gammastar = 0;          kCO2 = 995.4;

// General Et parameters
aE1 = 580;    aE2 = 5.1e-5;  aE3 = 0.3; aE4 = 0.01464;

// Other constants
cp = 1012; //specific heat air J kg-1 oC-1
sigmaSB = 4.9e-9; //stefan-Boltzmann MJ m-2 day-1 K-4
S0 = 117.5; //solar constant MJ m-2  day-1
bH1 =0.23;
bH2 =0.48;
mw = 2.99; //kg h2o MJ-1
alphaMS = 2; //mm oC-1 day-1 

var
{ List the variable names you are going to use here. Generally, this list
  includes all the symbols you defined in the procedure counts above. The order
  in which you list them does not matter. }
{States}
Cf, dCfdt,
Csw, dCswdt,
Chw, dChwdt,
Cr, dCrdt,
Nf, dNfdt, 
Nsw, dNswdt,
Nhw, dNhwdt,
Nr, dNrdt,
CLfw, dCLfwdt,
CLr, dCLrdt,
NLf, dNLfdt,
NLwr, dNLwrdt,
Cs, dCsdt,
Ce, dCedt,
Ci, dCidt, 
Ch, dChdt, 
Na, dNadt,
Nh, dNhdt,
Nnh4, dNnh4dt,
Nno3, dNno3dt,
WR, dWRdt,
WG, dWGdt,
WS, dWSdt,
Laieos, dLaieosdt,
Jday, dJdaydt,
DDa, dDDadt,
DDs, dDDsdt,
PptT, dPptTdt,
PptIT, dPptITdt,
EtT, dEtTdt,
DnT, dDntdt,
QgT, dQgTdt,
QwT, dQwTdt,
PST, dPSTdt,
NPPT, dNPPTdt,
tauNT, dtauNTdt,
RPT, dRPTdt,
lambdaCfT, dlambdaCfTdt,
lambdaCwT, dlambdaCwTdt,
lambdaCrT, dlambdaCrTdt,
lambdaNfT, dlambdaNfTdt,
lambdaNwT, dlambdaNwTdt,
lambdaNrT, dlambdaNrTdt,
lambdaCT, dlambdaCTdt,
lambdaNT, dlambdaNTdt,
RST, dRSTdt,
NEPT, dNEPdt,
UNT, dUNTdt,
qUnT, dqUnTdt,
UnfixT, dUnfixTdt,
NminT, dNminTdt,
QnT, dQnTdt,
Jdfs, dJdfsdt,
Jdws, dJdwsdt,
Jdrs, dJdrsdt,

{processes and associated parameters}
Ta, Ts,
Hd, Deltad,
Pptr, Tas,
Ppts,
PptI, bIsw, bIf,
Aw, Ww, Wf,
PsiD, Psiw, Psif, gsw,
ET, alphaT, gAW,
Ms, rhot, alphas, gammat,
DR, alphaD,
Qg, alphaQ,
Nnh4aq, rhoS, z, Vnh4max, etaNH4, Nno3aq, Vno3max, etaNO3,
Nnh4s, Nno3s,
QNH4, QNO3,
Cswmax, h1, h2,
gsC, omegaS,
LAI, mF,
Ps, aNf, Clfwmax, epsilonPs,
deltaPs, deltaCa, aDelta, bDelta,
tauN, atau,
HU, TUmax, TUopt, gammaU,
UNH4, UNH4max, kNH4, thetaU, Nlwrmax, epsilonUn,
UNO3, UNO3max, kNO3,
Unfix, qUn, gUn, qUnfix, gNfix, kNfix, RUnfix,
HR, TRmax, TRopt, gammaR,
RFmet, rm, qfmin, RWmet, awr, qwmin, RRmet, qrmin,
RFgr, rg, RWgr, RRgr,
fRLF, sL, fRLW, fRLR, RfmetL, RfmetS, RwmetL, RwmetS, RrmetL, RrmetS,
tauRnf, tauRnsw, tauRnr,
Tc, aLC, bLC, Tn, aLN, bLN,
FGrow, Jddstart, DDfstart, DDfstop,
WGrow, DDwstart, DDwstop,
RGrow, DDrstart, DDrstop,
Gcf, gf, bGf, epsilonF,
Gcsw, gw, bGw, epsilonW,
Gchw, aGHW,
Gcr, gr, bGr, epsilonR,
Gnf, Gnsw, Gnhw, Gnr,
lambdaCFD, alambdaFD, alambdaRD, Awd, atauD,
lambdaCRD, lambdaNFD, lambdaNRD, tauND,
FFall, JlambdaStart, TacrF, DsensF,
WFall, TacrW, DsensW,
RFall, TacrR, DsensR,
lambdaCF, alambdaF, sigmaLambdaF, Cfe,
lambdaCsw, alambdaW, sigmaLambdaW, Cwe,
lambdaChw,
lambdaCR, alambdaR, sigmaLambdaR, Cre,
lambdaNF, lambdaNsw, lambdaNhw, lambdaNR,
FClambdaE, FClambdaS, FClambdaI, FNlambdaA,
rSm, Q10D, sigmaD, Awdopt, Jdb,
FCSE, aSE,
FCEI, aEI,
FCIE, aIE,
FCIH, aIH,
RE, beta,
RH, aH,
MNA, aA,
MNH,
FNAH, qI,
INH4, VINH4max, kINH4,
INO3, VINO3max, kINO3,
Fntri, aNtri,
Cw, Nw, qF, qSW, qHW, qW, qR, qrs, CPT, CST, NPT, NST, qPT, qST,
NiT, Cet, NeT,
RF, RW, RR, Rp, Rs,
NPPf, NPPw, NPPr, NPP,
NEP,
lambdaC,
lambdaN,
Dn,
Un,
Mg,
Id,
Nmin,
Qn,

{drivers}
Ca,
Tamax,
Tamin,
Ppt,
II,
DNH4,
DNO3,
DlambdaCF, DlambdaCSW, DlambdaCHW, DlambdaCR,
DlambdaNF, DlambdaNSW, DlambdaNHW, DlambdaNR,
DNnh4, DNno3,

{other}
gamma, rhoa, Deltaes, esd, lambdav, Deltar, delta, PN, E0,
Cm, PD, PII, I0, HI, Ins, Tka, Tks, qu, Wdmin,Wdmax,
temp, Jd, Vc1, Vc2, Vn1, Vn2

:double;
npar, nproc, num:integer;

begin
{ Copy the drivers from the global array, drive, into the local variables. }
Ca := drive[1].value;
Tamax := drive[2].value;
Tamin := drive[3].value;
Ts := drive[4].value;
Ppt := drive[5].value;
II := drive[6].value;
DNH4 := drive[7].value;
DNO3 := drive[8].value;
DlambdaCF := drive[9].value;
DlambdaCSW := drive[10].value;
DlambdaCHW := drive[11].value;
DlambdaCR := drive[12].value;
DlambdaNF := drive[13].value;
DlambdaNSW := drive[14].value;
DlambdaNHW := drive[15].value;
DlambdaNR := drive[16].value;
DNnh4 := drive[17].value;
DNno3 := drive[18].value;

{ Copy the state variables from the global array into the local variables. }
Cf := tstat[1].value;
Csw := tstat[2].value;
Chw := tstat[3].value;
Cr := tstat[4].value;
Nf := tstat[5].value;
Nsw := tstat[6].value;
Nhw := tstat[7].value;
Nr := tstat[8].value;
CLfw := tstat[9].value;
CLr := tstat[10].value;
NLf := tstat[11].value;
NLwr := tstat[12].value;
Cs := tstat[13].value;
Ce := tstat[14].value;
Ci := tstat[15].value;
Ch := tstat[16].value;
Na := tstat[17].value;
Nh := tstat[18].value;
if DNnh4 = -1 then
  Nnh4 := tstat[19].value
else
   begin
     tstat[19].value:= DNnh4;
     Nnh4 := DNnh4;
   end;
if DNno3 = -1 then
  Nno3 := tstat[20].value
else
   begin
     tstat[20].value:= DNno3;
     Nno3 := DNno3;
   end;
WR := tstat[21].value;
WG := tstat[22].value;
WS := tstat[23].value;
Laieos := tstat[24].value;
Jday := tstat[25].value;
DDa := tstat[26].value;
DDs := tstat[27].value;
qUnT := tstat[28].value;
PptT := tstat[29].value;
PptIT := tstat[30].value;
EtT := tstat[31].value;
QgT := tstat[32].value;
QwT := tstat[33].value;
DnT := tstat[34].value;
PST := tstat[35].value;
NPPT := tstat[36].value;
tauNT := tstat[37].value;
RPT := tstat[38].value;
lambdaCfT := tstat[39].value;
lambdaCwT := tstat[40].value;
lambdaCrT := tstat[41].value;
lambdaNfT := tstat[42].value;
lambdaNwT := tstat[43].value;
lambdaNrT := tstat[44].value;
lambdaCT := tstat[45].value;
lambdaNT := tstat[46].value;
RST := tstat[47].value;
NEPT := tstat[48].value;
UNT := tstat[49].value;
UnfixT := tstat[50].value;
NminT := tstat[51].value;
QnT := tstat[52].value;
Jdfs := tstat[53].value;
Jdws := tstat[54].value;
Jdrs := tstat[55].value;

{ And now copy the parameters into the local variables. No need to copy the
  processes from the global array into local variables. Process values will be
  calculated by this procedure.

  Copy the parameters for each process separately using the function ParCount
  to keep track of the number of parameters in the preceeding processes.
  npar now contains the number of parameters in the preceding processes.
  copy the value of the first parameter of this process into it's local
  variable }

npar := ParCount(ModelDef.numstate + 3);
Deltad := par[npar + 1].value;

npar := ParCount(ModelDef.numstate + 4);
Tas := par[npar + 1].value;

npar := ParCount(ModelDef.numstate + 6);
bIf  := par[npar + 1].value;
bIsw := par[npar + 2].value;

npar := ParCount(ModelDef.numstate + 7);
Ww := par[npar + 1].value;
Wf := par[npar + 2].value;

npar := ParCount(ModelDef.numstate + 8);
Psiw := par[npar + 1].value;
Psif := par[npar + 2].value;

npar := ParCount(ModelDef.numstate + 10);
alphaT := par[npar + 1].value;
gAW := par[npar + 2].value;

npar := ParCount(ModelDef.numstate + 11);
rhot := par[npar + 1].value;
alphas := par[npar + 2].value;
gammat := par[npar + 3].value;

npar := ParCount(ModelDef.numstate + 12);
alphaD := par[npar + 1].value;

npar := ParCount(ModelDef.numstate + 13);
alphaQ := par[npar + 1].value;

npar := ParCount(ModelDef.numstate + 14);
rhoS := par[npar + 1].value;
z := par[npar + 2].value;
Vnh4max := par[npar + 3].value;
etaNH4 := par[npar + 4].value;

npar := ParCount(ModelDef.numstate + 15);
Vno3max := par[npar + 1].value;
etaNO3 := par[npar + 2].value;

npar := ParCount(ModelDef.numstate + 20);
h1 := par[npar + 1].value;
h2 := par[npar + 2].value;

npar := ParCount(ModelDef.numstate + 21);
omegaS := par[npar + 1].value;

npar := ParCount(ModelDef.numstate + 22);
mF := par[npar + 1].value;

npar := ParCount(ModelDef.numstate + 23);
aNf := par[npar + 1].value;
Clfwmax := par[npar + 2].value;
epsilonPs := par[npar + 3].value;

npar := ParCount(ModelDef.numstate + 24);
deltaCa := par[npar + 1].value;
aDelta := par[npar + 2].value;
bDelta := par[npar + 3].value;

npar := ParCount(ModelDef.numstate + 26);
atau := par[npar + 1].value;

npar := ParCount(ModelDef.numstate + 27);
TUmax := par[npar + 1].value;
TUopt := par[npar + 2].value;
gammaU := par[npar + 3].value;

npar := ParCount(ModelDef.numstate + 28);
UNH4max := par[npar + 1].value;
kNH4 := par[npar + 2].value;
thetaU := par[npar + 3].value;
Nlwrmax := par[npar + 4].value;
epsilonUn := par[npar + 5].value;

npar := ParCount(ModelDef.numstate + 29);
UNO3max := par[npar + 1].value;
kNO3 := par[npar + 2].value;

npar := ParCount(ModelDef.numstate + 30);
gUn := par[npar + 1].value;

npar := ParCount(ModelDef.numstate + 31);
qUnfix := par[npar + 1].value;
gNfix := par[npar + 2].value;
kNfix := par[npar + 3].value;

npar := ParCount(ModelDef.numstate + 33);
TRmax := par[npar + 1].value;
TRopt := par[npar + 2].value;
gammaR := par[npar + 3].value;

npar := ParCount(ModelDef.numstate + 34);
rm := par[npar + 1].value;
qFmin := par[npar + 2].value;

npar := ParCount(ModelDef.numstate + 35);
awr := par[npar + 1].value;
qWmin := par[npar + 2].value;

npar := ParCount(ModelDef.numstate + 36);
qRmin := par[npar + 1].value;

npar := ParCount(ModelDef.numstate + 37);
rg := par[npar + 1].value;

npar := ParCount(ModelDef.numstate + 40);
sL := par[npar + 1].value;

npar := ParCount(ModelDef.numstate + 52);
aLC := par[npar + 1].value;
bLC := par[npar + 2].value;

npar := ParCount(ModelDef.numstate + 53);
aLN := par[npar + 1].value;
bLN := par[npar + 2].value;

npar := ParCount(ModelDef.numstate + 54);
Jddstart := par[npar + 1].value;
DDfstart := par[npar + 2].value;
DDfstop := par[npar + 3].value;
JlambdaStart := par[npar + 4].value;
TacrF := par[npar + 5].value;
DsensF := par[npar + 6].value;

npar := ParCount(ModelDef.numstate + 55);
DDwstart := par[npar + 1].value;
DDwstop := par[npar + 2].value;
TacrW := par[npar + 3].value;
DsensW := par[npar + 4].value;

npar := ParCount(ModelDef.numstate + 56);
DDrstart := par[npar + 1].value;
DDrstop := par[npar + 2].value;
TacrR := par[npar + 3].value;
DsensR := par[npar + 4].value;

npar := ParCount(ModelDef.numstate + 60);
gf := par[npar + 1].value;
bGf := par[npar + 2].value;
epsilonF := par[npar + 3].value;

npar := ParCount(ModelDef.numstate + 61);
gw := par[npar + 1].value;
bGw := par[npar + 2].value;
epsilonW := par[npar + 3].value;

npar := ParCount(ModelDef.numstate + 62);
aGHW := par[npar + 1].value;

npar := ParCount(ModelDef.numstate + 63);
gr := par[npar + 1].value;
bGr := par[npar + 2].value;
epsilonR := par[npar + 3].value;

npar := ParCount(ModelDef.numstate + 68);
alambdaFD := par[npar + 1].value;
alambdaRD := par[npar + 2].value;
Awd := par[npar + 3].value;
atauD := par[npar + 4].value;

npar := ParCount(ModelDef.numstate + 75);
alambdaF := par[npar + 1].value;
sigmaLambdaF := par[npar + 2].value;
Cfe := par[npar + 3].value;

npar := ParCount(ModelDef.numstate + 76);
alambdaW := par[npar + 1].value;
sigmaLambdaW := par[npar + 2].value;
Cwe := par[npar + 3].value;

npar := ParCount(ModelDef.numstate + 78);
alambdaR := par[npar + 1].value;
sigmaLambdaR := par[npar + 2].value;
Cre := par[npar + 3].value;

npar := ParCount(ModelDef.numstate + 87);
Q10D := par[npar + 1].value;
sigmaD := par[npar + 2].value;
Awdopt := par[npar + 3].value;
Jdb := par[npar + 4].value;

npar := ParCount(ModelDef.numstate + 88);
aSE := par[npar + 1].value;

npar := ParCount(ModelDef.numstate + 89);
aEI := par[npar + 1].value;

npar := ParCount(ModelDef.numstate + 90);
aIE := par[npar + 1].value;

npar := ParCount(ModelDef.numstate + 91);
aIH := par[npar + 1].value;

npar := ParCount(ModelDef.numstate + 92);
beta := par[npar + 1].value;

npar := ParCount(ModelDef.numstate + 93);
aH := par[npar + 1].value;

npar := ParCount(ModelDef.numstate + 94);
aA := par[npar + 1].value;

npar := ParCount(ModelDef.numstate + 96);
qI := par[npar + 1].value;

npar := ParCount(ModelDef.numstate + 97);
VINH4max := par[npar + 1].value;
kINH4 := par[npar + 2].value;

npar := ParCount(ModelDef.numstate + 98);
VINO3max := par[npar + 1].value;
kINO3 := par[npar + 2].value;

npar := ParCount(ModelDef.numstate + 99);
aNtri := par[npar + 1].value;

{ Enter the equations to calculate the processes here, using the local variable
  names defined above. }

if newyear or (time=time_start) then    // Reset variables
begin
   Jday:= 0;
   DDa := 0;
   DDs := 0;
   if tstat[25].HoldConstant = False then tstat[25].value := Jday;
   if tstat[26].HoldConstant = False then tstat[26].value := DDa;
   if tstat[27].HoldConstant = False then tstat[27].value := DDs;
// reset water totals
   PptT := 0;
   PptIT := 0;
   EtT := 0;
   QgT := 0;
   QwT := 0;
   if tstat[29].HoldConstant = False then tstat[29].value := PptT;
   if tstat[30].HoldConstant = False then tstat[30].value := PptIT;
   if tstat[31].HoldConstant = False then tstat[31].value := EtT;
   if tstat[32].HoldConstant = False then tstat[32].value := QgT;
   if tstat[33].HoldConstant = False then tstat[33].value := QwT;
// reset N deposition, GPP, NPP, plant resp, and retranslocation
   DnT := 0;
   PST := 0;
   NPPT := 0;
   tauNT := 0;
   RPT := 0;
   if tstat[34].HoldConstant = False then tstat[34].value := DnT;
   if tstat[35].HoldConstant = False then tstat[35].value := PST;
   if tstat[36].HoldConstant = False then tstat[36].value := NPPT;
   if tstat[37].HoldConstant = False then tstat[37].value := tauNT;
   if tstat[38].HoldConstant = False then tstat[38].value := RPT;
// reset litter totals
   lambdaCfT := 0;
   lambdaCwT := 0;
   lambdaCrT := 0;
   lambdaNfT := 0;
   lambdaNwT := 0;
   lambdaNrT := 0;
   lambdaCT := 0;
   lambdaNT := 0;
   if tstat[39].HoldConstant = False then tstat[39].value := lambdaCfT;
   if tstat[40].HoldConstant = False then tstat[40].value := lambdaCwT;
   if tstat[41].HoldConstant = False then tstat[41].value := lambdaCrT;
   if tstat[42].HoldConstant = False then tstat[42].value := lambdaNfT;
   if tstat[43].HoldConstant = False then tstat[43].value := lambdaNwT;
   if tstat[44].HoldConstant = False then tstat[44].value := lambdaNrT;
   if tstat[45].HoldConstant = False then tstat[45].value := lambdaCT;
   if tstat[46].HoldConstant = False then tstat[46].value := lambdaNT;
// reset soil resp, NEPT, N uptake, n mineralization, and n leaching
   RST := 0;
   NEPT := 0;
   UNT := 0;
   UnfixT := 0;
   NminT := 0;
   QnT := 0;
   if tstat[47].HoldConstant = False then tstat[47].value := RST;
   if tstat[48].HoldConstant = False then tstat[48].value := NEPT;
   if tstat[49].HoldConstant = False then tstat[49].value := UNT;
   if tstat[50].HoldConstant = False then tstat[50].value := UnfixT;
   if tstat[51].HoldConstant = False then tstat[51].value := NminT;
   if tstat[52].HoldConstant = False then tstat[52].value := QnT;
// reset litter fall start
   if tstat[53].HoldConstant = False then tstat[53].value := 999;
   if tstat[54].HoldConstant = False then tstat[54].value := 999;
   if tstat[55].HoldConstant = False then tstat[55].value := 999;
end;

// Initialize discrete processes
Pptr := 0;
PptI := 0;
Ppts := 0;
Ms := 0;

// Temperature
Ta := (Tamax+Tamin)/2;

// LAI
LAI := Cf/mF;

// Sapwood
CSWmax := h1*LAIeos*power(Csw+Chw,h2);

// Available water
if Wr<Ww then Aw := 0
//else if Wr>Wf then Aw := 1
else Aw := (Wr-Ww)/(Wf-Ww);

// H2O tension deficit
if Aw>1 then PsiD:= Psif-PsiW else PsiD := -Psiw*(1-power(Psif/Psiw,Aw));
if PsiD <= 0 then PsiD:= 0.001;

// canopy H2O conductance and ET
gamma := 0.646*exp(0.00097*Tamax);
rhoa := 353/(273.2+Tamax);
Deltaes := 25029*exp(17.269*Tamax/(237.3+Tamax))/sqr(237.3+Tamax);
esd := 6.1078*(exp(17.269*Tamax/(237.3+Tamax))-exp(17.269*Tamin/(237.3+Tamin)));
lambdav := 2.501-2.37e-3*Tamax;
gsw := aE3*(LAI+0.1)*PsiD*exp(-aE4*esd);
ET := lambdav*(Deltaes+gamma*(1+gAW/gsW));
ET := (aE1*Deltaes*II*(1-alphaT)+aE2*rhoa*cp*esd*gAW)/ET;

// day length
Deltar := Deltad*pi/180;
delta := 0.1303*pi*cos(2*pi*(Jday-171.25)/365.25);
if tan(Deltar)*tan(delta) < -1 then
  Hd := 0
else if tan(Deltar)*tan(delta) > 1 then
  Hd := 24
else
  Hd := 24*arccos(-tan(Deltar)*tan(delta))/pi;

// canopy C conductance
gsC := power(abs(PsiD),x10)/(x6*omegas+0.5*(Tamax-Tamin));

// daily photosynthesis
PN := x1*aNf*NF*exp(x8*Tamax)/gsC;
PN:=PN/(1+power(CLfw/Clfwmax,epsilonPs));
E0 := x7*sqr(LAI)/(x9+sqr(LAI));
Cm := Ca+Gammastar-kCO2-PN;
Cm := Cm+sqrt(sqr(Cm)-4*(Ca*(gammastar-kco2)-PN*gammastar));
Cm := Cm/2;
PD := gsC*(Ca-Cm);
PII := Pd*E0*II/(PD+E0*II);
Ps := (x2*Hd+x5)*PII;
if Tamax <= 0 then Ps := 0;
deltaPs := deltaCa - aDelta-(bDelta - aDelta)*(Cm/Ca);

// DRAINAGE AND DISCHARGE
if (WR>WF) and (Ts>0) then DR:=alphaD*(WR-WF) else DR:= 0;
Qg := alphaQ*WG;

// Dissolved N
// aqueous NH4 and NO3
Nnh4aq := (Nnh4-rhoS*z*Vnh4max)/(1.4e-5*WR)-etaNH4;
Nnh4aq := 0.5*(Nnh4aq+sqrt(sqr(Nnh4aq)+4*etaNH4*Nnh4/(1.4e-5*WR)));
Nno3aq := (Nno3-rhoS*z*Vno3max)/(1.4e-5*WR)-etaNO3;
Nno3aq := 0.5*(Nno3aq+sqrt(sqr(Nno3aq)+4*etaNO3*Nno3/(1.4e-5*WR)));
// sorbed NH4 and NO3
Nnh4s := Nnh4-1.4e-5*WR*Nnh4aq;
Nno3s := Nno3-1.4e-5*WR*Nno3aq;
// N loss
Qnh4 := 1.4e-5*DR*Nnh4aq;
Qno3 := 1.4e-5*DR*Nno3aq;

// N uptake
if Ts<Tumax then
   begin
      HU := (Tumax-Ts)/(Tumax-Tuopt);
      HU := power(HU,gammaU*(Tumax-Tuopt));
      HU := HU*exp(gammaU*(Ts-Tuopt));
   end
else HU:= 0;
//qu := CR/NR; if qu < qRmin then qu := qRmin;

Unh4 := HU*Aw*Unh4max*(1-exp(-thetaU*CR))*Nnh4aq/((kNH4+Nnh4aq));
UNH4:=UNH4/(1+power(NLWR/Nlwrmax,epsilonUn));

Uno3 := HU*Aw*Uno3max*(1-exp(-thetaU*CR))*Nno3aq/((kNO3+Nno3aq));
UNo3:=UNo3/(1+power(NLWR/Nlwrmax,epsilonUn));

Un := Unh4 + Uno3;

qUn := gUn*II*exp(thetaU*Cr-0.5*LAI)*Ca/(kCO2+Ca);
qUn := qUn/((Unh4max*NNH4aq/(kNH4 + NNH4aq)) + (Uno3max*NNO3aq/(kNO3 + NNO3aq)));

// N fixation
Unfix := (1-exp(-sL*Clr))*HU*gNfix*Cr*(qUnT-qUnfix)/(knfix+(qUnT-qUnfix));
if qUnT < qUnfix then Unfix := 0;
RUnfix := qUnfix*Unfix;

// Drought Litterfall
if Aw <= Awd then
 begin
   lambdaCFD := alambdaFD*power((Awd - Aw),2)*Cf;
   lambdaCRD := alambdaRD*power((Awd - Aw),2)*Cr;
   lambdaNFD := (1-atauD)*lambdaCFD*Nf/Cf;
   lambdaNRD := lambdaCRD*Nr/Cr;
   tauND := atauD*lambdaCFD*Nf/Cf;
 end
else
 begin
   lambdaCFD := 0;
   lambdaCRD := 0;
   lambdaNFD := 0;
   lambdaNRD := 0;
   tauND := 0;
 end;

// Litterfall
if (Jday >= Jdfs) and (Jday < Jdfs + Dsensf) then
  FFall := 1
else
  FFall := 0;

if (Jday >= Jdws) and (Jday < Jdws + Dsensw) then
  WFall := 1
else
  WFall := 0;

if (Jday >= Jdrs) and (Jday < Jdrs + Dsensr) then
  RFall := 1
else
  RFall := 0;

if DlambdaCF = -1 then
  lambdaCF := FFall*alambdaF*CF*(1-Cfe/power(power(Cfe,sigmaLambdaF)+
                      power(CF,sigmaLambdaF),1/sigmaLambdaF)) + lambdaCFD
else
  lambdaCF := DlambdaCF;

Cw := Csw + Chw;
if DlambdaCsw = -1 then
 begin
  lambdaCsw := WFall*alambdaW*(Csw/Cw)*Cw*(1-Cwe/power(power(Cwe,sigmaLambdaW) +
                      power(Cw,sigmaLambdaW),1/sigmaLambdaW));
 end
else
 begin
  lambdaCsw := DlambdaCsw;
 end;

if DlambdaChw = -1 then
 begin
  lambdaChw := WFall*alambdaW*(Chw/Cw)*Cw*(1-Cwe/power(power(Cwe,sigmaLambdaW) +
                      power(Cw,sigmaLambdaW),1/sigmaLambdaW));
 end
else
 begin
  lambdaChw := DlambdaChw;
 end;

if DlambdaCR = -1 then
  lambdaCR := RFall*alambdaR*Cr*(1-Cre/power(power(Cre,sigmaLambdaR) +
                      power(Cr,sigmaLambdaR),1/sigmaLambdaR)) + lambdaCRD
else
  lambdaCR := DlambdaCR;

if DlambdaNF = -1 then
  lambdaNF := (1-atau)*lambdaCF*NF/CF + lambdaNFD
else
  lambdaNF := DlambdaNF;

if DlambdaNsw = -1 then
  lambdaNsw := lambdaCsw*NsW/CsW
else
  lambdaNsw := DlambdaNsw;

if Chw = 0 then
  lambdaNhw := 0
else if DlambdaNhw = -1 then
  lambdaNhw := lambdaChw*Nhw/Chw
else
  lambdaNhw := DlambdaNhw;

if DlambdaNR = -1 then
  lambdaNR := lambdaCR*NR/CR + lambdaNRD
else
  lambdaNR := DlambdaNR;

// Translocation
tauN := atau*lambdaCF*NF/CF + tauND;

// Growth
if (DDfstart <= DDa) and (DDa <= DDfstop) then
  FGrow := 1
else
  FGrow := 0;

if (DDwstart <= DDa) and (DDa <= DDwstop) then
  WGrow := 1
else
  WGrow := 0;

if (DDrstart <= DDs) and (DDs <= DDrstop) then
  RGrow := 1
else
  RGrow := 0;

if (Clfw > 0) and (Nlf > 0) then
 begin
   Gcf := FGrow*gF*Cf*(Clfw/(Cf+Csw))*power(Nlf/Cf,1-epsilonF);
   Gnf := FGrow*(gF/bgf)*Cf*power(Clfw/(Cf+Csw),1-epsilonF)*Nlf/Cf;
{   if (CF>mF*LAIeos) and (GCF>1) then
      begin
         GNF:=GNF*1/GCf;
         GCf:=1;
      end;  }
 end
else
 begin
   Gcf :=0;
   Gnf :=0;
 end;

if (Clfw > 0) and (Nlwr > 0) then
 begin
   Gcsw := WGrow*gw*Csw*(Clfw/(Cf+Csw))*power(NlWR/(Csw+Cr),1-epsilonW);
   Gnsw := WGrow*(gw/bgw)*Csw*power(Clfw/(Cf+Csw),1-epsilonW)*Nlwr/(Csw+Cr);
 end
else
 begin
   Gcsw :=0;
   Gnsw :=0;
 end;

if (Clr > 0) and (Nlwr > 0) then
 begin
   Gcr := RGrow*gr*Cr*(Clr/Cr)*power(Nlwr/(Csw+Cr),1-epsilonR);
   Gnr := RGrow*(gr/bgr)*Cr*power(Clr/Cr,1-epsilonR)*Nlwr/(Csw+Cr);
 {  if (CF>mF*LAIeos) and (GCr>1) then
      begin
         GNr:=GNr*1/GCr;
         GCr:=1;
      end;      }
 end
else
 begin
   Gcr := 0;
   Gnr := 0;
 end;

if GCF < 0 then GCF := 0;
if GCSW < 0 then GCSW := 0;
if GCR < 0 then GCR := 0;
if GNF < 0 then GNF := 0;
if GNSW < 0 then GNSW := 0;
if GNR < 0 then GNR := 0;

if Csw >= Cswmax then
  Gchw := WGrow*aGHW*(Csw-Cswmax)
else
  Gchw := 0;
Gnhw := Gchw*Nsw/Csw;

// Respiration
RFgr := rg*GCF;
RWgr := rg*GCSW;
RRgr := rg*GCR;

if Ta<TRmax then
   begin
      HR := (TRmax-Ta)/(Trmax-Tropt);
      HR := power(HR,gammaR*(TRmax-TRopt));
      HR := HR*exp(gammaR*(Ta-TRopt));
   end
else HR:= 0;

if CF/NF>qFmin then RFmet := rm*HR*NF else RFmet := rm*HR*CF/qFmin;
if CsW/NsW>qWmin then RWmet := rm*aWR*HR*NSW else RWmet := rm*aWR*HR*CSW/qWmin;

if Ts<TRmax then
   begin
      HR := (TRmax-Ts)/(Trmax-Tropt);
      HR := power(HR,gammaR*(TRmax-TRopt));
      HR := HR*exp(gammaR*(Ts-TRopt));
   end
else HR:= 0;

if CR/NR>qRmin then RRmet := rm*HR*NR else RRmet := rm*HR*CR/qRmin;

fRLF := 1-exp(-sL*Clfw);
fRLW := 1-exp(-sL*Clfw);
fRLR := 1-exp(-sL*Clr);
RfmetL := Rfmet*fRLF;
RwmetL := Rwmet*fRLW;
//if RfmetL + RwmetL > Clfw
RrmetL := Rrmet*fRLR;
RfmetS := Rfmet*(1-fRLF);
RwmetS := Rwmet*(1-fRLW);
RrmetS := Rrmet*(1-fRLR);

qF := Cf/Nf;
qSW := Csw/Nsw;
if Nhw <> 0 then qHW := Chw/Nhw else qHW := 0;
qW := (Csw+Chw)/(Nsw+Nhw);
qR := Cr/Nr;

tauRnf := RfmetS/qF;
tauRnsw := RwmetS/qSW;
tauRnr := RrmetS/qR;

// Allocation (Transport)
Tc := aLC*Laieos*((Clfw/(Cf+Csw))/bLC-CLr/Cr)/power(Cw,h2);
Tn := aLN*Laieos*((NLwr/(Csw+Cr))/bLN-NLf/Cf)/power(Cw,h2);

// Litter partitioning
FClambdaE := 0;
FClambdaS := 0;
if lambdaCF <> 0 then
 begin
   FClambdaE := (0.49 - 0.452*exp(-95*lambdaNF/lambdaCF))*lambdaCF;
   FClambdaS := (0.4 + 0.39*exp(-155*lambdaNF/lambdaCF))*lambdaCF;
 end;
if lambdaCSW <> 0 then
 begin
   FClambdaE := FClambdaE + (0.49 - 0.452*exp(-95*lambdaNSW/lambdaCSW))*lambdaCSW;
   FClambdaS := FClambdaS + (0.4 + 0.39*exp(-155*lambdaNSW/lambdaCSW))*lambdaCSW;
 end;
if lambdaChw <> 0 then
 begin
   FClambdaE := FClambdaE + (0.49 - 0.452*exp(-95*lambdaNhw/lambdaChw))*lambdaChw;
   FClambdaS := FClambdaS + (0.4 + 0.39*exp(-155*lambdaNhw/lambdaChw))*lambdaChw;
 end;
if lambdaCR <> 0 then
 begin
   FClambdaE := FClambdaE + (0.49 - 0.452*exp(-95*lambdaNR/lambdaCR))*lambdaCR;
   FClambdaS := FClambdaS + (0.4 + 0.39*exp(-155*lambdaNR/lambdaCR))*lambdaCR;
 end;
FClambdaI := lambdaCF + lambdaCSW + lambdaChw + lambdaCR - FClambdaE - FClambdaS;
FNlambdaA := lambdaNF + lambdaNSW + lambdaNhw + lambdaNR;

// Soil metabolic rate
Jd := 1-sqr(Aw-Awdopt)/(sqr(Jdb)*(sigmaD*(Aw-Awdopt)+1));
if Jd < 0 then Jd := 0;
rsm := Jd*power(Q10D,Ts/10);

// Soil fluxes
FCSE := rSm*aSE*Cs;
FCEI := rSm*aEI*CE;
//FCIE := rSm*aIE*CI;
FCIE := min(rSm*aIE*CI,rSm*aIE*Na*qI);
FCIH := min(rsm*aIH*Ci,rsm*aIH*Na*qI);
RE := beta*(FCSE+FCEI+FCIE);
RH := rSm*aH*CH;
MNA := rSm*aA*NA;
MNH := rSm*aH*NH;
FNAH := FCIH/qI;
if CI/qI>NA then INH4:=rSm*VINH4max*Nnh4aq*(CI/qI - NA)/(kINH4+Nnh4aq)
else INH4 := 0;
if CI/qI>NA then INO3:=rSm*VINO3max*Nno3aq*(CI/qI - NA)/(kINO3+Nno3aq)
else INO3 := 0;
Fntri := rsm*aNtri*Nnh4aq;

// Summaries
Cw := Csw + Chw;
Nw := Nsw + Nhw;
Cpt := Cf + Csw + Chw + Cr + Clfw + Clr;
Npt := Nf + Nsw + Nhw + Nr + Nlf + Nlwr;
Cst := Cs + Ce + Ci + Ch;
Nst := Na + Nh;
qPT := Cpt/Npt;
qST := Cst/Nst;
Nit := Nnh4 + Nno3;
Cet := Cpt + Cst;
Net := Npt + Nst;
qrs := Cr/Cf;
RF := RFmet + RFgr;
RW := RWmet + RWgr;
RR := RRmet + RRgr;
Rp := RF + RW + RR + RUNfix;
Rs := Re + Rh;
NPPF := GCF;
NPPW := GCSW;
NPPR := GCR;
NPP := Ps - Rp;
NEP := NPP - Rs;
lambdaC := lambdaCF + lambdaCSW + lambdaChw + lambdaCR;
lambdaN := lambdaNF + lambdaNSW + lambdaNhw + lambdaNR;
Dn := Dnh4 + Dno3;
Mg := Mna + Mnh;
Id := Inh4 + Ino3;
Nmin := Mna + Mnh - Inh4 - Ino3;
Qn := Qnh4 + Qno3;


if CalculateDiscrete then
begin
 // End of year?
 if trunc(time) mod 366 = 0 then
   newyear := true
 else
   newyear := false;

 // increment Julian day
 dJdaydt := 1*time_step;

 // degree day
 if (Jday >= Jddstart) then
  begin
    if Ta > 0 then dDDadt := Ta*time_step else dDDadt:=0;
    if Ts > 0 then dDDsdt := Ts*time_step else dDDsdt:=0;
  end
 else
  begin
    dDDadt := 0;
    dDDsdt := 0;
  end;
  
 // laieos
 if DDa = DDfstop then LAIeos := 0;
 if (DDa >= DDfstop) and (Jday < Jdfs) then
   Laieos := Lai;  //max(Laieos, Lai);

 // Litterfall season
 if (Jday + dJdaydt >= Jlambdastart) and (Jdfs = 999) then
   begin
    if Ta <= TacrF then tstat[53].value := Jday + dJdaydt;
   end;
 if (Jday + dJdaydt >= Jlambdastart) and (Jdws = 999) then
   begin
    if Ta <= TacrW then tstat[54].value := Jday + dJdaydt;
   end;
 if (Jday + dJdaydt >= Jlambdastart) and (Jdrs = 999) then
   begin
    if Ts <= TacrR then tstat[55].value := Jday + dJdaydt;
   end;

 // Cumulative N deposition
 dDnTdt := Dn*time_step;

 // Rain and Snow
 If Ta<Tas then
   begin
      Pptr := 0;
      PPts := Ppt;
      PptI := 0;
   end
 else
   begin
      Pptr := Ppt;
      PPts := 0;
      PptI := bIsw*CSW + bIf*LAI;
      if PptI > Pptr then PptI := Pptr;
   end;

 // Snowmelt
 I0 := S0*(Hd*sin(Deltar)*sin(delta)/24 + cos(Deltar)*sin(Hd*pi/24)/pi);
 if I0 <> 0 then HI := (II/I0 - bH1)/bH2 else HI := 0;
 Tka := Ta + 273.15;
 if Ta>2.5 then Tks := 273.15 else Tks := Tka-2.5;
 Ins := 0.1 + 0.9*HI;
 Ins := Ins*(0.76*power(Tka,4) - power(Tks,4))*(1 - rhot);
 Ins := Ins + rhot*(power(Tka,4) - power(Tks,4));
 Ins := II*exp(-gammat*rhot)*(1 - alphaS) + sigmaSB*Ins;
 MS := 0;
 if Ins > 0 then MS := MS + mw*INS;
 if Ta > 0 then MS := MS + alphaMS*Ta;
 if MS > WS then MS := WS;

 // Water flux derivatives
 dWsdt := (Ppts - Ms)*time_step;
 dWrdt := (Pptr - PptI + Ms)*time_step;
 dPptTdt := Ppt*time_step;
 dPptITdt := PptI*time_step;
 dQwTdt := dPptITdt;

 // Update state variables directly because these are discrete processes and
 // don't pass through the integrator.
 if tstat[21].HoldConstant = False then tstat[21].value := Wr + dWrdt;
 if tstat[23].HoldConstant = False then tstat[23].value := Ws + dWsdt;
 if tstat[24].HoldConstant = False then tstat[24].value := Laieos;
 if tstat[25].HoldConstant = False then tstat[25].value := Jday + dJdaydt;
 if tstat[26].HoldConstant = False then tstat[26].value := DDa + dDDadt;
 if tstat[27].HoldConstant = False then tstat[27].value := DDs + dDDsdt;
 if tstat[29].HoldConstant = False then tstat[29].value := PptT + dPptTdt;
 if tstat[30].HoldConstant = False then tstat[30].value := PptIT + dPptITdt;
 if tstat[33].HoldConstant = False then tstat[33].value := QwT + dQwTdt;
 if tstat[34].HoldConstant = False then tstat[34].value := DnT + dDnTdt;
end; //discrete processes

{ Now calculate the derivatives of the state variables. If the holdConstant
  portion of the state variable is set to true then set the derivative equal to
  zero. }
if tstat[1].HoldConstant = False then
 dCfdt := Gcf - lambdaCF - Rfmets
else
 dCfdt := 0;

if tstat[2].HoldConstant = False then
 dCswdt := Gcsw - lambdaCSW - Gchw - Rwmets
else
 dCswdt := 0;

if tstat[3].HoldConstant = False then
 dChwdt := Gchw - lambdaChW
else
 dChwdt := 0;

if tstat[4].HoldConstant = False then
 dCrdt := Gcr - lambdaCR - Rrmets
else
 dCrdt := 0;

if tstat[5].HoldConstant = False then
 dNfdt := Gnf - lambdaNF - tauN - tauRnf
else
 dNfdt := 0;

if tstat[6].HoldConstant = False then
 dNswdt := Gnsw - Gnhw - lambdaNSW - tauRnsw
else
 dNswdt := 0;

if tstat[7].HoldConstant = False then
 dNhwdt := Gnhw - lambdaNHW
else
 dNhwdt := 0;

if tstat[8].HoldConstant = False then
 dNrdt := Gnr - lambdaNR - tauRnr
else
 dNrdt := 0;

if tstat[9].HoldConstant = False then
 dCLfwdt := Ps - Tc - RfmetL - RwmetL - Rfgr - Rwgr - Gcf - Gcsw
else
 dCLfwdt := 0;

if tstat[10].HoldConstant = False then
 dCLrdt := Tc - RrmetL - Rrgr - Gcr - RUnfix
else
 dCLrdt := 0;

if tstat[11].HoldConstant = False then
 dNLfdt := Tn + tauRnf - Gnf
else
 dNLfdt := 0;

if tstat[12].HoldConstant = False then
 dNLwrdt := UN + Unfix + tauN + tauRnsw + tauRnr - Tn - Gnsw - Gnr
else
 dNLwrdt := 0;

if tstat[13].HoldConstant = False then
 dCsdt := FClambdaS - Fcse
else
 dCsdt := 0;

if tstat[14].HoldConstant = False then
 dCedt := FClambdaE + Fcse + Fcie - Re - Fcei
else
 dCedt := 0;

if tstat[15].HoldConstant = False then
 dCidt := FClambdaI + Fcei - Fcie - Fcih
else
 dCidt := 0;

if tstat[16].HoldConstant = False then
 dChdt := Fcih - Rh
else
 dChdt := 0;

if tstat[17].HoldConstant = False then
 dNadt := FNlambdaA + Inh4 + Ino3 - Fnah - Mna
else
 dNadt := 0;

if tstat[18].HoldConstant = False then
 dNhdt := Fnah - Mnh
else
 dNhdt := 0;

if tstat[19].HoldConstant = False then
 dNnh4dt := Dnh4 + Mna + Mnh - Inh4 - Unh4 - Fntri - Qnh4
else
 dNnh4dt := 0;

if tstat[20].HoldConstant = False then
 dNno3dt := Dno3 + Fntri - Uno3 - Ino3 - Qno3
else
 dNno3dt := 0;

if tstat[21].HoldConstant = False then
 dWRdt := - Et - Dr // Note: Pptr, PptI, and Ms are discrete processes so they
else        // are not included here. The state variable is updated with changes
 dWRdt := 0;  // due to these processes directly during the calculate discrete call.

if tstat[22].HoldConstant = False then
 dWGdt := Dr - Qg
else
 dWGdt := 0;

if tstat[23].HoldConstant = False then
 dWSdt := 0   // Function of discrete processes so state variable is updated directly.
else
 dWSdt := 0;

if tstat[24].HoldConstant = False then
 dLaieosdt := 0  // Function of discrete processes so state variable is updated directly.
else
 dLaieosdt := 0; // Function of discrete processes so state variable is updated directly.

if tstat[25].HoldConstant = False then
 dJdaydt := 0  // Function of discrete processes so state variable is updated directly.
else
 dJdaydt := 0; // Function of discrete processes so state variable is updated directly.

if tstat[26].HoldConstant = False then
 dDDadt := 0  // Function of discrete processes so state variable is updated directly.
else
 dDDadt := 0;

if tstat[27].HoldConstant = False then
 dDDsdt := 0  // Function of discrete processes so state variable is updated directly.
else
 dDDsdt := 0;

if tstat[28].HoldConstant = False then
 dqUnTdt := (qUn-qUnT)/365
else
 dqUnTdt := 0;

if tstat[29].HoldConstant = False then
 dPptTdt := 0 // Function of discrete processes so state variable is updated directly.
else
 dPptTdt := 0;

if tstat[30].HoldConstant = False then
 dPptITdt := 0 // Function of discrete processes so state variable is updated directly.
else
 dPptITdt := 0;

if tstat[31].HoldConstant = False then
 dEtTdt := Et
else
 dEtTdt := 0;

if tstat[32].HoldConstant = False then
 dQgTdt := Qg
else
 dQgTdt := 0;

if tstat[33].HoldConstant = False then
 dQwTdt := Et + Qg
else
 dQwTdt := 0;

if tstat[34].HoldConstant = False then
 dDnTdt := 0 // Function of discrete processes so state variable is updated directly.
else
 dDnTdt := 0;

if tstat[35].HoldConstant = False then
 dPSTdt := Ps
else
 dPSTdt := 0;

if tstat[36].HoldConstant = False then
 dNPPTdt := NPP
else
 dNPPTdt := 0;

if tstat[37].HoldConstant = False then
 dtauNTdt := tauN
else
 dtauNTdt := 0;

if tstat[38].HoldConstant = False then
 dRPTdt := Rp
else
 dRPTdt := 0;

if tstat[39].HoldConstant = False then
 dlambdaCfTdt := lambdaCf
else
 dlambdaCfTdt := 0;

if tstat[40].HoldConstant = False then
 dlambdaCwTdt := lambdaCsw + lambdaChw
else
 dlambdaCwTdt := 0;

if tstat[41].HoldConstant = False then
 dlambdaCrTdt := lambdaCr
else
 dlambdaCrTdt := 0;

if tstat[42].HoldConstant = False then
 dlambdaNfTdt := lambdaNf
else
 dlambdaNfTdt := 0;

if tstat[43].HoldConstant = False then
 dlambdaNwTdt := lambdaNsw + lambdaNhw
else
 dlambdaNwTdt := 0;

if tstat[44].HoldConstant = False then
 dlambdaNrTdt := lambdaNr
else
 dlambdaNrTdt := 0;

if tstat[45].HoldConstant = False then
 dlambdaCTdt := lambdaC
else
 dlambdaCTdt := 0;

if tstat[46].HoldConstant = False then
 dlambdaNTdt := lambdaN
else
 dlambdaNTdt := 0;

if tstat[47].HoldConstant = False then
 dRSTdt := Rs
else
 dRSTdt := 0;

if tstat[48].HoldConstant = False then
 dNEPdt := NEP
else
 dNEPdt := 0;

if tstat[49].HoldConstant = False then
 dUNTdt := Un
else
 dUNTdt := 0;

if tstat[50].HoldConstant = False then
 dUnfixTdt := Unfix
else
 dUnfixTdt := 0;

if tstat[51].HoldConstant = False then
 dNminTdt := Nmin
else
 dNminTdt := 0;

if tstat[52].HoldConstant = False then
 dQnTdt := Qn
else
 dQnTdt := 0;

if tstat[53].HoldConstant = False then
 dJdfsdt := 0  // Function of discrete processes so state variable is updated directly.
else
 dJdfsdt := 0;

if tstat[54].HoldConstant = False then
 dJdwsdt := 0  // Function of discrete processes so state variable is updated directly.
else
 dJdwsdt := 0;

if tstat[55].HoldConstant = False then
 dJdrsdt := 0  // Function of discrete processes so state variable is updated directly.
else
 dJdrsdt := 0;


{ Now that the calculations are complete, assign everything back into the arrays
  so the rest of the code can access the values calculated here. (Local variables
  are destroyed at the end of the procedure).

  Put all process values into process variable array. The first numstate
  processes are the derivatives of the state variables. }
tproc[1].value := dCfdt;
tproc[2].value := dCswdt;
tproc[3].value := dChwdt;
tproc[4].value := dCrdt;
tproc[5].value := dNfdt;
tproc[6].value := dNswdt;
tproc[7].value := dNhwdt;
tproc[8].value := dNrdt;
tproc[9].value := dCLfwdt;
tproc[10].value := dCLrdt;
tproc[11].value := dNLfdt;
tproc[12].value := dNLwrdt;
tproc[13].value := dCsdt;
tproc[14].value := dCedt;
tproc[15].value := dCidt;
tproc[16].value := dChdt;
tproc[17].value := dNadt;
tproc[18].value := dNhdt;
tproc[19].value := dNnh4dt;
tproc[20].value := dNno3dt;
tproc[21].value := dWRdt;
tproc[22].value := dWGdt;
tproc[23].value := dWSdt;
tproc[24].value := dLaieosdt;
tproc[25].value := dJdaydt;
tproc[26].value := dDDadt;
tproc[27].value := dDDsdt;
tproc[28].value := dqUnTdt;
tproc[29].value := dPptTdt;
tproc[30].value := dPptITdt;
tproc[31].value := dEtTdt;
tproc[32].value := dQgTdt;
tproc[33].value := dQwTdt;
tproc[34].value := dDnTdt;
tproc[35].value := dPSTdt;
tproc[36].value := dNPPTdt;
tproc[37].value := dtauNTdt;
tproc[38].value := dRPTdt;
tproc[39].value := dlambdaCfTdt;
tproc[40].value := dlambdaCwTdt;
tproc[41].value := dlambdaCrTdt;
tproc[42].value := dlambdaNfTdt;
tproc[43].value := dlambdaNwTdt;
tproc[44].value := dlambdaNrTdt;
tproc[45].value := dlambdaCTdt;
tproc[46].value := dlambdaNTdt;
tproc[47].value := dRSTdt;
tproc[48].value := dNEPdt;
tproc[49].value := dUNTdt;
tproc[50].value := dUnfixTdt;
tproc[51].value := dNminTdt;
tproc[52].value := dQnTdt;
tproc[53].value := dJdfsdt;
tproc[54].value := dJdwsdt;
tproc[55].value := dJdrsdt;

{ Now the remaining processes. Be sure to number the processes the same here as
  you did in the procedure counts above. }

tproc[ModelDef.numstate + 1].value := Ta;
tproc[ModelDef.numstate + 2].value := Ts;
tproc[ModelDef.numstate + 3].value := Hd;
tproc[ModelDef.numstate + 4].value := Pptr;
tproc[ModelDef.numstate + 5].value := Ppts;
tproc[ModelDef.numstate + 6].value := PptI;
tproc[ModelDef.numstate + 7].value := Aw;
tproc[ModelDef.numstate + 8].value := PsiD;
tproc[ModelDef.numstate + 9].value := gsw;
tproc[ModelDef.numstate + 10].value := Et;
tproc[ModelDef.numstate + 11].value := Ms;
tproc[ModelDef.numstate + 12].value := DR;
tproc[ModelDef.numstate + 13].value := Qg;
tproc[ModelDef.numstate + 14].value := Nnh4aq;
tproc[ModelDef.numstate + 15].value := Nno3aq;
tproc[ModelDef.numstate + 16].value := Nnh4s;
tproc[ModelDef.numstate + 17].value := Nno3s;
tproc[ModelDef.numstate + 18].value := Qnh4;
tproc[ModelDef.numstate + 19].value := Qno3;
tproc[ModelDef.numstate + 20].value := Cswmax;
tproc[ModelDef.numstate + 21].value := gsC;
tproc[ModelDef.numstate + 22].value := LAI;
tproc[ModelDef.numstate + 23].value := Ps;
tproc[ModelDef.numstate + 24].value := deltaPs;
tproc[ModelDef.numstate + 25].value := tauND;
tproc[ModelDef.numstate + 26].value := tauN;
tproc[ModelDef.numstate + 27].value := HU;
tproc[ModelDef.numstate + 28].value := UNH4;
tproc[ModelDef.numstate + 29].value := UNO3;
tproc[ModelDef.numstate + 30].value := qUn;
tproc[ModelDef.numstate + 31].value := Unfix;
tproc[ModelDef.numstate + 32].value := RUnfix;
tproc[ModelDef.numstate + 33].value := Hr;
tproc[ModelDef.numstate + 34].value := RFmet;
tproc[ModelDef.numstate + 35].value := RWmet;
tproc[ModelDef.numstate + 36].value := RRmet;
tproc[ModelDef.numstate + 37].value := RFgr;
tproc[ModelDef.numstate + 38].value := RWgr;
tproc[ModelDef.numstate + 39].value := RRgr;
tproc[ModelDef.numstate + 40].value := fRLF;
tproc[ModelDef.numstate + 41].value := fRLW;
tproc[ModelDef.numstate + 42].value := fRLR;
tproc[ModelDef.numstate + 43].value := RfmetL;
tproc[ModelDef.numstate + 44].value := RwmetL;
tproc[ModelDef.numstate + 45].value := RrmetL;
tproc[ModelDef.numstate + 46].value := RfmetS;
tproc[ModelDef.numstate + 47].value := RwmetS;
tproc[ModelDef.numstate + 48].value := RrmetS;
tproc[ModelDef.numstate + 49].value := tauRnf;
tproc[ModelDef.numstate + 50].value := tauRnsw;
tproc[ModelDef.numstate + 51].value := tauRnr;
tproc[ModelDef.numstate + 52].value := Tc;
tproc[ModelDef.numstate + 53].value := Tn;
tproc[ModelDef.numstate + 54].value := 0;
tproc[ModelDef.numstate + 55].value := 0;
tproc[ModelDef.numstate + 56].value := 0;
tproc[ModelDef.numstate + 57].value := FGrow;
tproc[ModelDef.numstate + 58].value := WGrow;
tproc[ModelDef.numstate + 59].value := RGrow;
tproc[ModelDef.numstate + 60].value := Gcf;
tproc[ModelDef.numstate + 61].value := Gcsw;
tproc[ModelDef.numstate + 62].value := Gchw;
tproc[ModelDef.numstate + 63].value := Gcr;
tproc[ModelDef.numstate + 64].value := Gnf;
tproc[ModelDef.numstate + 65].value := Gnsw;
tproc[ModelDef.numstate + 66].value := Gnhw;
tproc[ModelDef.numstate + 67].value := Gnr;
tproc[ModelDef.numstate + 68].value := lambdaCFD;
tproc[ModelDef.numstate + 69].value := lambdaCRD;
tproc[ModelDef.numstate + 70].value := lambdaNFD;
tproc[ModelDef.numstate + 71].value := lambdaNRD;
tproc[ModelDef.numstate + 72].value := FFall;
tproc[ModelDef.numstate + 73].value := WFall;
tproc[ModelDef.numstate + 74].value := RFall;
tproc[ModelDef.numstate + 75].value := lambdaCF;
tproc[ModelDef.numstate + 76].value := lambdaCSW;
tproc[ModelDef.numstate + 77].value := lambdaCHW;
tproc[ModelDef.numstate + 78].value := lambdaCR;
tproc[ModelDef.numstate + 79].value := lambdaNF;
tproc[ModelDef.numstate + 80].value := lambdaNSW;
tproc[ModelDef.numstate + 81].value := lambdaNHW;
tproc[ModelDef.numstate + 82].value := lambdaNR;
tproc[ModelDef.numstate + 83].value := FClambdaE;
tproc[ModelDef.numstate + 84].value := FClambdaS;
tproc[ModelDef.numstate + 85].value := FClambdaI;
tproc[ModelDef.numstate + 86].value := FNlambdaA;
tproc[ModelDef.numstate + 87].value := rSm;
tproc[ModelDef.numstate + 88].value := FCSE;
tproc[ModelDef.numstate + 89].value := FCEI;
tproc[ModelDef.numstate + 90].value := FCIE;
tproc[ModelDef.numstate + 91].value := FCIH;
tproc[ModelDef.numstate + 92].value := RE;
tproc[ModelDef.numstate + 93].value := RH;
tproc[ModelDef.numstate + 94].value := MNA;
tproc[ModelDef.numstate + 95].value := MNH;
tproc[ModelDef.numstate + 96].value := FNAH;
tproc[ModelDef.numstate + 97].value := INH4;
tproc[ModelDef.numstate + 98].value := INO3;
tproc[ModelDef.numstate + 99].value := Fntri;
tproc[ModelDef.numstate + 100].value := Cw;
tproc[ModelDef.numstate + 101].value := Nw;
tproc[ModelDef.numstate + 102].value := qF;
tproc[ModelDef.numstate + 103].value := qSW;
tproc[ModelDef.numstate + 104].value := qHW;
tproc[ModelDef.numstate + 105].value := qW;
tproc[ModelDef.numstate + 106].value := qR;
tproc[ModelDef.numstate + 107].value := qrs;
tproc[ModelDef.numstate + 108].value := CPT;
tproc[ModelDef.numstate + 109].value := CST;
tproc[ModelDef.numstate + 110].value := NPT;
tproc[ModelDef.numstate + 111].value := NST;
tproc[ModelDef.numstate + 112].value := qPT;
tproc[ModelDef.numstate + 113].value := qST;
tproc[ModelDef.numstate + 114].value := Nit;
tproc[ModelDef.numstate + 115].value := Cet;
tproc[ModelDef.numstate + 116].value := Net;
tproc[ModelDef.numstate + 117].value := RF;
tproc[ModelDef.numstate + 118].value := RW;
tproc[ModelDef.numstate + 119].value := RR;
tproc[ModelDef.numstate + 120].value := Rp;
tproc[ModelDef.numstate + 121].value := Rs;
tproc[ModelDef.numstate + 122].value := NPPF;
tproc[ModelDef.numstate + 123].value := NPPW;
tproc[ModelDef.numstate + 124].value := NPPR;
tproc[ModelDef.numstate + 125].value := NPP;
tproc[ModelDef.numstate + 126].value := NEP;
tproc[ModelDef.numstate + 127].value := lambdaC;
tproc[ModelDef.numstate + 128].value := lambdaN;
tproc[ModelDef.numstate + 129].value := Dn;
tproc[ModelDef.numstate + 130].value := Un;
tproc[ModelDef.numstate + 131].value := Mg;
tproc[ModelDef.numstate + 132].value := Id;
tproc[ModelDef.numstate + 133].value := Nmin;
tproc[ModelDef.numstate + 134].value := Qn;

end;  // End of processes procedure


       { Do not make any modifications to code below this line. }
{****************************************************************************}


{This function counts the parameters in all processes less than processnum.}
function ParCount(processnum:integer) : integer;
var
 NumberofParams, counter : integer;
begin
  NumberofParams := 0;
  for counter := ModelDef.numstate + 1 to processnum - 1 do
         NumberofParams := NumberofParams + proc[counter].parameters;
  ParCount := NumberofParams;
end; // end of parcount function

{ This procedure supplies the derivatives of the state variables to the
  integrator. Since the integrator deals only with the values of the variables
  and not there names, units or the state field HoldConstant, this procedure
  copies the state values into a temporary state array and copies the value of
  HoldConstant into the temporary state array and passes this temporary state
  array to the procedure processes. }
PROCEDURE derivs(t:double; var tdrive:drivearray; var tpar:paramarray;
             var statevalue:yValueArray; VAR dydt:yValueArray);
var
   i:integer;
   tempproc:processarray;
   tempstate:statearray;
begin
   tempstate := stat;  // Copy names, units and HoldConstant to tempstate
  // Copy current values of state variables into tempstate
   for i := 1 to ModelDef.numstate do tempstate[i].value := statevalue[i];
  // Calculate the process values
   processes(t,tdrive,tpar,tempstate,tempproc,false);
  // Put process values into dydt array to get passed back to the integrator.
   for i:= 1 to ModelDef.numstate do dydt[i]:=tempproc[i].value;
end;  // end of derivs procedure

end.
